from weakref import ref
import random
from PySide6.QtGui import QPen, QPainter, QColor, QPixmap, QIcon
from PySide6.QtCore import Qt, QVariantAnimation, QRectF, QEasingCurve, QSize
from PySide6.QtWidgets import QApplication, QPushButton

from qtmui.material.styles import useTheme


class Arc:
    def __init__(self):
        theme = useTheme()
        self.diameter = 40  # Đường kính vòng cung
        self.color = QColor(theme.palette.grey._500)
        self.span = random.randint(40, 150)  # Độ dài cung ban đầu
        self.direction = 1 if random.randint(10, 15) % 2 == 0 else -1
        self.startAngle = random.randint(0, 360)


class LoadingIcon(QIcon):
    def __init__(self, button: QPushButton, diameter=40, parent=None):
        """
        Biểu tượng loading dưới dạng QIcon.

        Parameters:
        - button: QPushButton, nút sẽ hiển thị biểu tượng.
        - diameter: int, đường kính của vòng cung.
        """
        super().__init__(parent)
        self.button_ref = ref(button)  # Nút để cập nhật biểu tượng
        self.arc = Arc()
        self.arc.diameter = diameter
        self.pixmap_size = diameter + 10  # Kích thước QPixmap (để chứa vòng cung)
        self.startAnime()

    def startAnime(self):
        """Khởi động animation quay vòng."""
        self.anim = QVariantAnimation(duration=1000)
        self.anim.setStartValue(0)
        self.anim.setEndValue(360)
        self.anim.setEasingCurve(QEasingCurve.Linear)
        self.anim.setLoopCount(-1)  # Lặp lại vô hạn
        self.anim.valueChanged.connect(self.updatePixmap)
        self.anim.start()

    def updatePixmap(self, value):
        """
        Cập nhật QPixmap theo góc quay của animation.
        """
        # Tạo QPixmap trống
        pixmap = QPixmap(self.pixmap_size, self.pixmap_size)
        pixmap.fill(Qt.transparent)  # Nền trong suốt

        # Vẽ vòng cung lên QPixmap
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.setPen(QPen(self.arc.color, 3, Qt.SolidLine))

        # Tính toán tọa độ và độ dài cung
        current_angle = value
        start_angle = current_angle * self.arc.direction + self.arc.startAngle
        rect = QRectF(
            5, 5, self.arc.diameter, self.arc.diameter
        )  # Vùng chứa vòng cung

        # Điều chỉnh độ dài cung thay đổi từ 40 -> 240
        if current_angle <= 180:
            span_length = 40 + (current_angle / 180) * 200
        else:
            span_length = 240 - ((current_angle - 180) / 180) * 200

        # Vẽ vòng cung
        painter.drawArc(rect, int(start_angle) * 16, int(span_length) * 16)
        painter.end()

        # Cập nhật QIcon bằng QPixmap
        self.addPixmap(pixmap)

        # Kiểm tra xem nút còn tồn tại trước khi cập nhật
        try:
            button = self.button_ref()
            if button:
                button.setIcon(self)
        except Exception as e:
            self.anim.valueChanged.disconnect(self.updatePixmap)

    def stopAnimation(self):
        """Dừng animation."""
        if self.anim:
            self.anim.stop()


# if __name__ == "__main__":

#     app = QApplication(sys.argv)

#     # Tạo một QPushButton và sử dụng LoadingIcon làm biểu tượng
#     button = QPushButton("Loading Button")
#     loading_icon = LoadingIcon(button, diameter=40)  # Truyền tham chiếu tới nút
#     button.setIconSize(QSize(loading_icon.pixmap_size, loading_icon.pixmap_size))
#     button.resize(200, 100)
#     button.show()

#     sys.exit(app.exec())



from PySide6.QtCore import Qt, Property, Slot, Signal
from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel, QHBoxLayout
from qtmui.hooks import State

from ..flow_layout import FlowLayout
from qtmui.material.styles import useTheme

from ..controller import Controller
from ..textfield import TextField
from ..autocomplete.complete import Autocomplete
from ..form_control_label import FormControlLabel
from ..checkbox import Checkbox, MultiCheckbox
from ..form_control import FormControl
from ..form_label import FormLabel
from ..box import Box
from ..form_group import FormGroup
from ..form_helper_text import FormHelperText

from ..qss_name import *

class RHFCheckbox(QFrame):
    valueChanged = Signal(object)

    def __init__(
            self,
            name: str,
            value: object = None,
            checked: bool = False,
            label: str = None,
            error: bool = False,
            helperText: str = None,
            ):
        super().__init__()

        self._name = name
        self._value = value

        self._stateSignal = None

        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        # self.setStyleSheet('background: red;')

        field = {
            "value": "checkbox value demooooooo"
        }

        control = False

        if checked:
            self._value = True
        else:
            self._value = False


        self.lbl_helper_text = QLabel(self)

        self.layout().addWidget(
            Controller(
                name=name,
                control=control,
                render=Box(
                    children=[
                        FormControlLabel(
                            label=label, 
                            # hightLight=True,
                            checked=field.get("value"),
                            control=Checkbox(
                                checked=checked, onChange=self.onChange,
                            )
                        ),
                        self.lbl_helper_text
                    ]
                )

            )
        )

    def onChange(self, value):
        self._value = value
        print(f"Field width name {self._name} has value {self._value}")
        self.valueChanged.emit(self._value)

    @Property(bool)
    def stateSignal(self):
        return self._stateSignal

    @stateSignal.setter
    def stateSignal(self, value):
        self._stateSignal = value
        self._stateSignal.connect(self.state)


    @Slot(object)
    def state(self, state):
        theme = useTheme()
        if self._name == state.get("field"):
            if state.get("error"):
                # self._complete._inputField._set_slot({"slot": "error", "message": state.get("error_message")})
                self.lbl_helper_text.setText(str(state.get("error_message")[0]))
                self.lbl_helper_text.setStyleSheet(f'''
                    padding-left: 8px;
                    font-size: {theme.typography.caption.fontSize};
                    font-weight: {theme.typography.caption.fontWeight};
                    color: {theme.palette.error.main};
                ''')
                if not self.lbl_helper_text.isVisible():
                    self.lbl_helper_text.show()
            else:
                # self._complete._inputField._set_slot({"slot": "valid"})
                self.lbl_helper_text.hide()


class RHFMultiCheckbox(QFrame):
    valueChanged = Signal(object)

    def __init__(
            self,
            row: bool = False,
            name: str = None,
            label: str = None,
            options: object = None,
            spacing: int = None,
            helperText: str = None,
            value: object = None,
            sx: dict = None,
            *other
            ):
        super().__init__()
        self.setLayout(QVBoxLayout())

        self._name = name
        self._value = value
        control = False
        error = False

        self._stateSignal = None

        self.lbl_helper_text = QLabel(self)

        self.multiCheckbox = MultiCheckbox(options=options, value=value, onChange=self._on_change)

        self.layout().addWidget(
            Controller(
                name=name,
                control=control,
                render=FormControl(
                    children=[
                        FormLabel(component="legend", label=label or ""),
                        FormGroup(
                            children=[
                                self.multiCheckbox
                            ]
                        ),
                        self.lbl_helper_text
                    ]
                )
            )
        )


    def _set_value(self, value: list):
        for option in self.findChildren(Checkbox):
            option.setChecked(option.text() in value)

    def _on_change(self, value):
        self._value = value
        print(f"Field width name {self._name} has value {self._value}")
        self.valueChanged.emit(value)

    @Property(bool)
    def stateSignal(self):
        return self._stateSignal

    @stateSignal.setter
    def stateSignal(self, value):
        self._stateSignal = value
        self._stateSignal.connect(self.state)


    @Slot(object)
    def state(self, state):
        theme = useTheme()
        if self._name == state.get("field"):
            if state.get("error"):
                # self._complete._inputField._set_slot({"slot": "error", "message": state.get("error_message")})
                self.lbl_helper_text.setText(str(state.get("error_message")[0]))
                self.lbl_helper_text.setStyleSheet(f'''
                    padding-left: 8px;
                    font-size: {theme.typography.caption.fontSize};
                    font-weight: {theme.typography.caption.fontWeight};
                    color: {theme.palette.error.main};
                ''')
                if not self.lbl_helper_text.isVisible():
                    self.lbl_helper_text.show()
            else:
                # self._complete._inputField._set_slot({"slot": "valid"})
                self.lbl_helper_text.hide()


class RHFMultiCheckboxNew(QWidget):
    def __init__(
            self,
            row: bool = False,
            flowLayout: bool = False,
            name: str = None,
            label: str = None,
            options: object = None,
            spacing: int = None,
            helperText: str = None,
            sx: dict = None,
            *other
            ):
        super().__init__()

        if flowLayout:
            self.flow_layout = FlowLayout(self)
            self.setLayout(self.flow_layout)
        else:
            if row:
                self.setLayout(QHBoxLayout())
            else:
                self.setLayout(QVBoxLayout())

        field = {
            "value": "multi checkbox value"
        }
        control = False
        error = False

        self.layout().addWidget(
            Controller(
                name=name,
                control=control,
                render=FormControl(
                    children=[
                        FormLabel(component="legend",  label=label),
                        FormGroup(
                            row=row,
                            # sx={{
                            # ...(row && {
                            #     flexDirection: 'row',
                            # }),
                            # [`& .${formControlLabelClasses.root}`]: {
                            #     '&:not(:last-of-type)': {
                            #     mb: spacing || 0,
                            #     },
                            #     ...(row && {
                            #     mr: 0,
                            #     '&:not(:last-of-type)': {
                            #         mr: spacing || 2,
                            #     },
                            #     }),
                            # },
                            # ...sx,
                            # }}
                            children=[
                                FormControlLabel(
                                    key=option["value"],
                                    control=Checkbox(
                                        checked=option["value"] in field["value"],
                                        onChange=lambda: field.on_change(self.get_selected(field["value"], option["value"]))
                                    ),
                                    label=option["label"],
                                    *other
                                ) for option in options
                            ]
                        ),
                        FormHelperText(error=bool(error), sx={'mx': 0}, children=error.message if error else helperText) if error or helperText else None

                    ]
                )
            )
        )

    def get_selected(self, field_value, option_value):
        pass



import uuid
from typing import Callable, Optional, Union
from PySide6.QtCore import Qt, Signal, Slot, Property
from PySide6.QtWidgets import QWidget, QVBoxLayout, QLabel

from qtmui.hooks import useState
from qtmui.hooks import State

from ..controller import Controller
from ..textfield import TextField

from qtmui.material.styles import useTheme

from qtmui.qss_name import *

class RHFTextField(QWidget):
    valueChanged = Signal(object)

    def __init__(
            self,
            name: str,
            key: str=None,
            label: Optional[Union[str, State, Callable]]=None,
            value: str = None,
            defaultValue: str = None,
            type: object = None,
            field: object = None,
            multiline: bool = False,
            options: object = None,
            spacing: int = None,
            InputProps: object = None,
            helperText: str = None,
            # textChanged: object = None,
            placeholder: str = None,
            *args,
            **kwargs
            ):
        super().__init__()

        self._name = name
        self._key = key
        self._value = value
        self._InputProps = InputProps
        self._placeholder = placeholder

        self._state, self._setState = useState(None)

        self._label = label
        self._defaultValue = defaultValue
        
        if self._defaultValue is not None:
            self._value = self._defaultValue
            
        self._stateSignal = None

        self._field = field
        control = False
        error = None
        self._type = type

        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.layout().setSpacing(0)
        self.setObjectName(str(uuid.uuid4()))
        # self.setStyleSheet('''#{}  {{ {} }}'''.format(self.objectName(), "border: none;background: pink;")) # str multi line

        if not multiline:
            self.setFixedHeight(60)


        self.textField = TextField(
                    key=self._key,
                    label=self._label,
                    value=self._value,
                    type=self._type,
                    defaultValue=self._defaultValue,
                    multiline=multiline,
                    InputProps=self._InputProps,
                    placeholder=self._placeholder,
                    # value="" if self._type == "number" and field.value == 0 else field.value,
                    # textChanged=lambda event: field.textChanged(int(event.target.value)) if type == "number" else field.textChanged(event.target.value),
                    # onChange=textChanged,
                    error=not error,
                    # helperText=error.message if error and hasattr(error, 'message') else helperText
                    *args,
                    **kwargs
                )

        self.layout().addWidget(
            Controller(
                name=name,
                control=control,
                render=self.textField
            )
        )

        self.lbl_helper_text = QLabel(self)
        self.lbl_helper_text.setFixedHeight(14)
        self.lbl_helper_text.setStyleSheet(f'''
            padding-left: 8px;
            {useTheme().typography.caption.to_qss_props()}
            font-size: 11px;
        ''')
        self.lbl_helper_text.setText("")
        self.layout().addWidget(self.lbl_helper_text)

        self.textField.valueChanged.connect(self.set_only_value_not_change_text)


    def set_only_value_not_change_text(self, value):
        if self._type == "int":
            self._value = int(value)
        elif self._type == "float":
            self._value = float(value)
        else:
            self._value = value

        self.valueChanged.emit(self._value)
        self._setState(self._value)
        # print('RHFTextField_setState_________________________1111111111', self._state.value, self._value)
        

    def set_value(self, value, setText=True):
        if self._type == "int":
            self._value = int(value)
        elif self._type == "float":
            self._value = float(value)
        else:
            self._value = value

        self.valueChanged.emit(self._value)

        self.textField._set_data(value=self._value, valueChanged=False)
        
        self._setState(self._value)
        # print('RHFTextField_setState_________________________222222222', self._state.value, self._value)


    @Property(bool)
    def stateSignal(self):
        return self._stateSignal

    @stateSignal.setter
    def stateSignal(self, value):
        self._stateSignal = value
        self._stateSignal.connect(self.state)


    @Slot(object)
    def state(self, state):
        theme = useTheme()
        if self._name == state.get("field"):
            if state.get("error"):
                self.textField._set_slot({"slot": "error", "message": state.get("error_message")})
                self.lbl_helper_text.setText(str(state.get("error_message")[0]))
                self.lbl_helper_text.setStyleSheet(f'''
                    padding-left: 8px;
                    font-size: {theme.typography.caption.fontSize};
                    font-weight: {theme.typography.caption.fontWeight};
                    color: {theme.palette.error.main};
                ''')
                if not self.lbl_helper_text.isVisible():
                    self.lbl_helper_text.show()
            else:
                self.textField._set_slot({"slot": "valid"})
                self.lbl_helper_text.hide()



# src/qtmui/hooks/use_effect.py
import inspect
from typing import Callable, List, Optional
from PySide6.QtCore import QTimer, QObject
from .use_state import State
from ._effect_context import register_effect
from ._effect_runner import _EffectRunner  # sẽ tạo file này


def useEffect(callback: Callable, dependencies: List[State]) -> None:
    # Lấy component hiện tại từ call stack
    frame = inspect.currentframe().f_back
    component = frame.f_locals.get('self')
    if not component or not isinstance(component, QObject):
        raise RuntimeError("useEffect must be called inside a Section/QObject component")

    runner = _EffectRunner(callback, dependencies)
    register_effect(component, runner)


def useLayoutEffect(callback: Callable, dependencies: List[State]) -> None:
    def wrapped():
        QTimer.singleShot(0, callback)
    useEffect(wrapped, dependencies)


from dataclasses import dataclass, field, is_dataclass, asdict, replace
from typing import Literal, Optional, Dict, Any, Callable, Union

import threading
from immutable import Immutable
import uuid
from redux import ReducerResult, CompleteReducerResult, BaseAction, BaseEvent, Store
from typing import Callable, Sequence, Optional

from PySide6.QtCore import QPoint, QSize
from PySide6.QtGui import QColor

from qtmui.utils.lodash import merge, dataclass_to_dict

from .shape import ShapeOptions, Shape
from .create_breakpoints import BreakpointsOptions, Breakpoints
from .create_spacing import SpacingOptions, Spacing, create_spacing
from ..style_function_sx.style_function_sx import SxProps
from ..style_function_sx.default_sx_config import SxConfig
from .apply_styles import ApplyStyles
from .typography import Typography
from .palette import palette
from .create_palette import create_palette, Palette
from .create_size import create_size, Sizes
from .create_shadows import create_shadows, Shadows
from .create_root_component_styles import create_root_component_styles


class Direction:
    ltr = "ltr"
    rtl = "rtl"
# Direction = Union['ltr', 'rtl']


# ThemeOptions dataclass equivalent
# @dataclass
class ThemeOptions(Immutable):
    shape: Optional[ShapeOptions] = None
    breakpoints: Optional[BreakpointsOptions] = None
    direction: Optional[Direction] = 'ltr'
    mixins: Optional[Any] = None
    palette: Optional[Dict[str, Any]] = field(default_factory=dict)
    customShadows: Optional[Shadows] = None
    spacing: Optional[SpacingOptions] = None
    createSpacing: Optional[Callable] = None
    transitions: Optional[Any] = None
    components: Optional[Dict[str, Any]] = field(default_factory=dict)
    typography: Optional[Any] = None
    zIndex: Optional[Dict[str, int]] = field(default_factory=dict)
    unstable_sxConfig: Optional[SxConfig] = None
    size: Optional[Dict[str, Any]] = field(default_factory=dict)

class Context(Immutable):
    mainwindow_pos: Optional[ShapeOptions] = None
    mainwindow_size: Optional[ShapeOptions] = None

class Changed(Immutable):
    connect: Optional[object] = None

class Signal(Immutable):
    changed: Optional[Changed] = None

# Cập nhật ThemeState để sử dụng Palette thay vì Dict
# @dataclass(frozen=True)
class ThemeState(Immutable):
    state: Optional[Signal] = None
    context: Context
    shape: Shape
    breakpoints: Breakpoints
    direction: Direction
    palette: Palette  # palette giờ là class Palette thay vì Dict
    customShadows: Optional[Shadows] = None
    spacing: Spacing = field(default_factory=Spacing)
    transitions: Optional[Any] = None
    components: Optional[Dict[str, Any]] = field(default_factory=dict)
    mixins: Optional[Any] = None
    typography: Typography = None
    zIndex: Optional[Dict[str, int]] = field(default_factory=dict)
    applyStyles: Optional[Callable[[str], ApplyStyles]] = None
    unstable_sxConfig: Optional[SxConfig] = None
    unstable_sx: Optional[Callable[[SxProps], Dict[str, Any]]] = None
    size: Optional[Sizes] = create_size()


class CreateThemeAction(BaseAction):
    pass

class ChangePaletteAction(BaseAction):
    mode: Literal['light', 'dark']  # Chỉ cho phép 'light' hoặc 'dark'

class MergeOverideComponentsAction(BaseAction):
    payload: Dict

class UpdateMainwindowPositionAction(BaseAction):
    mainWindowPosition: QPoint

def createTheme(options: Optional[ThemeOptions] = None, *args: Any) -> ThemeState:
    options = options or ThemeOptions()
    
    # Create Shape, Breakpoints, and other components
    context = Context()
    shape = Shape() if options.shape is None else options.shape
    breakpoints = Breakpoints() if options.breakpoints is None else options.breakpoints
    
    palette = options.palette or {"mode": "light"}
    direction = options.direction or "ltr"
    customShadows = options.customShadows
    spacing = Spacing() if options.spacing is None else options.spacing
    typography = options.typography
    zIndex = options.zIndex or {}
    components = options.components or {}
    
    # Create and return the full Theme object
    return ThemeState(
        context=context,
        shape=shape,
        breakpoints=breakpoints,
        direction=direction,
        palette=palette,
        customShadows=customShadows,
        spacing=spacing,
        typography=typography,
        zIndex=zIndex,
        components=components,
        applyStyles=None,  # Can be updated later
        unstable_sxConfig=options.unstable_sxConfig,
        unstable_sx=None  # Can be updated later
    )

def theme_reducer(
    state: ThemeState | None,
    action: BaseAction,
) -> ReducerResult[ThemeState, BaseAction, BaseEvent]:
    if state is None:
        _palette = create_palette(palette("light"))
        state = createTheme(ThemeOptions(
            shape=ShapeOptions(borderRadius=4),
            breakpoints=BreakpointsOptions(),
            palette=_palette,
            typography=Typography(),
            # createSpacing=create_spacing(),  # Sử dụng đối tượng Spacing thay vì SpacingOptions
            spacing=Spacing(),  # Sử dụng đối tượng Spacing thay vì SpacingOptions
            size=create_size(),
            customShadows=create_shadows(_palette)
        ))
        root_component_styles = create_root_component_styles(state)
        return replace(state, components=root_component_styles)


    if isinstance(action, CreateThemeAction):
        pass
    if isinstance(action, ChangePaletteAction):
        _palette = create_palette(palette(action.mode))
        state = replace(
                state, 
                palette=_palette,
                customShadows=create_shadows(_palette)
            )
        root_component_styles = create_root_component_styles(state)
        # print('root_component_stylesf____________', root_component_styles)
        return replace(state, components=root_component_styles)

    if isinstance(action, MergeOverideComponentsAction):
        _palette = action.payload.get("palette")
        _components = action.payload.get("components")
        if _components and isinstance(_components, dict):
            state = replace(state, components=_components)
        if _palette and isinstance(_palette, dict):
            current_palette_dict = dataclass_to_dict(state.palette)
            # merge sâu
            merged_palette = merge(current_palette_dict, _palette)
            # tái tạo lại Palette object
            new_palette = create_palette(merged_palette)
            # print('new_palettet__', merged_palette)
            state = replace(state, palette=new_palette)
    
    if isinstance(action, UpdateMainwindowPositionAction):
        return replace(state, context=replace(state.context, mainwindow_pos=action.mainWindowPosition))

    return state



from __future__ import annotations

from typing import Callable, Optional, Union, List, Dict
import math

from PySide6.QtWidgets import QWidget, QVBoxLayout
from PySide6.QtGui import QPainter, QColor, QBrush, QPen, QFont, QPainterPath
from PySide6.QtCore import Qt, QPointF, QRectF

from qtmui.hooks import State
from qtmui.material.styles import useTheme


class ChartPolarArea(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        series: Optional[Union[List[Dict], List[float]]] = None,
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: Optional[Dict] = None,
        key: str = None,
        title: Optional[Union[State, str, Callable]] = None,
        *args,
        **kwargs,
    ):
        """
        Nếu `series` là danh sách dict thì mong đợi có các key: "label", "value".
        Nếu là danh sách số thì dùng thêm options để truyền danh sách nhãn.
        """
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self._options = options or {}
        self._title = title

        if width:
            self.setFixedWidth(width)
        if height:
            self.setFixedHeight(height)

        # Xử lý dữ liệu đầu vào
        if series and all(isinstance(item, dict) for item in series):
            self.values = [item.get("value", 0) for item in series]
            self.labels = [item.get("label", f"Category {i+1}") for i, item in enumerate(series)]
        elif series and all(isinstance(item, (int, float)) for item in series):
            self.values = series
            self.labels = self._options.get("labels", [f"Category {i+1}" for i in range(len(series))])
        else:
            self.values = []
            self.labels = []

        # Các màu sắc cho từng phân vùng, dùng options nếu có
        self.colors = self._options.get("colors", [
            "#ADD8E6",  # xanh lam nhạt
            "#FFFF00",  # vàng
            "#90EE90",  # xanh lá cây nhạt
            "#00008B",  # xanh lam đậm
            "#00CED1",  # xanh ngọc lam
            "#FFA500",  # cam
            "#006400",  # xanh lá cây đậm
            "#A52A2A",  # nâu
        ])
        # Màu stroke cho đường viền (mặc định dùng màu nền của theme)
        self.stroke_color = self._options.get("stroke", {}).get("colors", [useTheme().palette.background.paper])[0]
        # Độ mờ của màu fill (opacity từ 0 đến 1)
        self.fill_opacity = self._options.get("fill", {}).get("opacity", 0.8)
        # Giá trị tối đa hiển thị trên trục (theo mô tả: 25)
        self.max_value = 25

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        full_rect = self.rect()

        # Dành không gian cho biểu đồ và legend:
        legend_width = 120  # độ rộng dành cho legend bên phải
        chart_rect = full_rect.adjusted(0, 0, -legend_width, 0)

        center = QPointF(chart_rect.width() / 2, chart_rect.height() / 2)
        # Dịch chuyển tâm chart theo vị trí của chart_rect
        center.setX(center.x() + chart_rect.left())
        center.setY(center.y() + chart_rect.top())

        outer_radius = min(chart_rect.width(), chart_rect.height()) * 0.45

        # 1. Vẽ các vòng tròn đồng tâm (tick: 5, 10, 15, 20, 25)
        pen = QPen(QColor(self.stroke_color))
        pen.setStyle(Qt.DashLine)
        painter.setPen(pen)
        font = QFont()
        font.setPointSize(8)
        painter.setFont(font)
        ticks = [5, 10, 15, 20, 25]
        for tick in ticks:
            r = (tick / self.max_value) * outer_radius
            painter.drawEllipse(center, r, r)
            # Vẽ nhãn tick bên phải vòng tròn
            painter.drawText(QPointF(center.x() + r + 2, center.y()), f"{tick}")

        # 2. Vẽ các trục (spokes)
        n = len(self.values)
        if n == 0:
            return
        sector_angle = 360 / n
        pen_spoke = QPen(QColor(self.stroke_color))
        pen_spoke.setStyle(Qt.SolidLine)
        painter.setPen(pen_spoke)
        for i in range(n):
            angle_deg = i * sector_angle
            angle_rad = math.radians(angle_deg)
            x = center.x() + outer_radius * math.cos(angle_rad)
            y = center.y() - outer_radius * math.sin(angle_rad)
            painter.drawLine(center, QPointF(x, y))
            # Vẽ nhãn cho mỗi trục (chú thích của danh mục) ngay ngoài biên chart
            label = self.labels[i] if i < len(self.labels) else f"Category {i+1}"
            label_x = center.x() + (outer_radius + 10) * math.cos(angle_rad)
            label_y = center.y() - (outer_radius + 10) * math.sin(angle_rad)
            painter.drawText(QPointF(label_x, label_y), label)

        # 3. Vẽ các phân vùng màu (Polar Area Chart kiểu Nightingale)
        for i, value in enumerate(self.values):
            r_value = (value / self.max_value) * outer_radius
            angle_center = i * sector_angle
            half_sector = sector_angle / 2
            start_angle = angle_center - half_sector
            end_angle = angle_center + half_sector

            # Tạo đường viền (path) cho sector
            path = QPainterPath()
            path.moveTo(center)
            start_rad = math.radians(start_angle)
            start_point = QPointF(
                center.x() + r_value * math.cos(start_rad),
                center.y() - r_value * math.sin(start_rad)
            )
            path.lineTo(start_point)
            # Vẽ cung với các điểm trung gian
            steps = 30
            for step in range(1, steps + 1):
                interp_angle = start_angle + (end_angle - start_angle) * (step / steps)
                interp_rad = math.radians(interp_angle)
                pt = QPointF(
                    center.x() + r_value * math.cos(interp_rad),
                    center.y() - r_value * math.sin(interp_rad)
                )
                path.lineTo(pt)
            path.lineTo(center)
            path.closeSubpath()

            # Lấy màu theo thứ tự, áp dụng độ mờ
            color = QColor(self.colors[i % len(self.colors)])
            color.setAlphaF(self.fill_opacity)
            painter.fillPath(path, QBrush(color))
            # Vẽ viền cho sector
            painter.setPen(QPen(QColor(self.stroke_color)))
            painter.drawPath(path)

        # 4. Vẽ tiêu đề (nếu có)
        if self._title:
            title_font = QFont()
            title_font.setBold(True)
            title_font.setPointSize(10)
            painter.setFont(title_font)
            painter.drawText(chart_rect.adjusted(10, 10, -10, -10), Qt.AlignTop | Qt.AlignHCenter, self._title)

        # 5. Vẽ legend (chú thích màu) bên phải biểu đồ
        legend_rect = QRectF(chart_rect.right() + 10, chart_rect.top(), legend_width - 10, full_rect.height())
        legend_font = QFont()
        legend_font.setPointSize(8)
        painter.setFont(legend_font)

        # Đặt màu văn bản cho legend rõ ràng (sử dụng màu text của theme)
        text_color = QColor(useTheme().palette.text.primary)
        
        swatch_size = 12
        spacing = 4
        y_offset = legend_rect.top() + 20  # vị trí bắt đầu vẽ legend

        for i, label in enumerate(self.labels):
            # Chọn màu từ danh sách colors
            color = QColor(self.colors[i % len(self.colors)])
            # Vẽ hình vuông màu (swatch)
            swatch_rect = QRectF(legend_rect.left(), y_offset, swatch_size, swatch_size)
            painter.fillRect(swatch_rect, QBrush(color))
            painter.setPen(QPen(QColor(self.stroke_color)))
            painter.drawRect(swatch_rect)
            # Tạo bounding rect cho text legend
            text_rect = QRectF(swatch_rect.right() + spacing, y_offset, legend_rect.width() - swatch_size - spacing, swatch_size)
            value_text = f"{self.values[i]}" if i < len(self.values) else ""
            full_text = f"{label}: {value_text}"
            painter.setPen(text_color)
            painter.drawText(text_rect, Qt.AlignLeft | Qt.AlignVCenter, full_text)
            y_offset += swatch_size + spacing + 4

        painter.end()




from dataclasses import is_dataclass


def merge(*dict_args)-> dict:
    result = {}
    for dictionary in dict_args:
        result.update(dictionary)
    return result

def dataclass_to_dict(obj):
    """Đệ quy chuyển dataclass Palette và các lớp con thành dict."""
    if is_dataclass(obj):
        result = {}
        for k, v in obj.__dict__.items():
            if callable(v):
                # bỏ qua các function
                continue
            result[k] = dataclass_to_dict(v)
        return result
    elif isinstance(obj, dict):
        return {k: dataclass_to_dict(v) for k, v in obj.items()}
    elif isinstance(obj, (list, tuple)):
        return [dataclass_to_dict(v) for v in obj]
    else:
        return obj


from dataclasses import dataclass


@dataclass
class TypographyType:
    fontWeight: int
    lineHeight: float
    fontSize: str
    responsiveSizes: dict
    textTransform: str


class TypographyStyle:
    """Lớp để định nghĩa kiểu chữ với các thuộc tính như fontSize, fontWeight, lineHeight."""

    def __init__(self, fontWeight=None, lineHeight=None, fontSize=None, responsiveSizes=None, textTransform=None):
        self.fontWeight = fontWeight or 400
        self.lineHeight = lineHeight
        self.fontSize = fontSize
        self.responsiveSizes = responsiveSizes or {}
        self.textTransform = textTransform or "unset"

    def __repr__(self):
        return f"TypographyStyle(fontWeight={self.fontWeight}, lineHeight={self.lineHeight}, fontSize={self.fontSize})".encode('utf-8').decode('utf-8')
    
    def to_qss_props(self):
        return f"""
        font-weight: {self.fontWeight};
        line-height: {self.lineHeight}em;
        font-size: {self.fontSize};
        """

    def toQFont(self):
        from PySide6.QtGui import QFont

        font = QFont()
        font.setPointSizeF(int(self.fontSize.replace('px', '')))
        font.setWeight(QFont.Weight(self.fontWeight))
        return font

class Typography:
    h1: TypographyType
    h2: TypographyType
    h3: TypographyType
    h4: TypographyType
    h5: TypographyType
    h6: TypographyType
    subtitle1: TypographyType
    subtitle2: TypographyType
    body1: TypographyType
    body2: TypographyType
    caption: TypographyType
    overline: TypographyType
    button: TypographyType

    def __init__(self):
        self.primary_font = "Public Sans, sans-serif"
        self.fontWeightRegular = 400
        self.fontWeightMedium = 500
        self.fontWeightSemiBold = 600
        self.fontWeightBold = 700

        self.h1 = TypographyStyle(
            fontWeight=800,
            lineHeight=80 / 64,
            fontSize=self.int_to_px(40),
            responsiveSizes=self.responsive_font_sizes(sm=52, md=58, lg=64)
        )
        self.h2 = TypographyStyle(
            fontWeight=800,
            lineHeight=64 / 48,
            fontSize=self.int_to_px(32),
            responsiveSizes=self.responsive_font_sizes(sm=40, md=44, lg=48)
        )
        self.h3 = TypographyStyle(
            fontWeight=700,
            lineHeight=1.5,
            fontSize=self.int_to_px(24),
            responsiveSizes=self.responsive_font_sizes(sm=26, md=30, lg=32)
        )
        self.h4 = TypographyStyle(
            fontWeight=700,
            lineHeight=1.5,
            fontSize=self.int_to_px(20),
            responsiveSizes=self.responsive_font_sizes(sm=20, md=24, lg=24)
        )
        self.h5 = TypographyStyle(
            fontWeight=700,
            lineHeight=1.5,
            fontSize=self.int_to_px(18),
            responsiveSizes=self.responsive_font_sizes(sm=19, md=20, lg=20)
        )
        self.h6 = TypographyStyle(
            fontWeight=700,
            lineHeight=28 / 18,
            fontSize=self.int_to_px(17),
            responsiveSizes=self.responsive_font_sizes(sm=18, md=18, lg=18)
        )
        self.subtitle1 = TypographyStyle(
            fontWeight=600,
            lineHeight=1.5,
            fontSize=self.int_to_px(16)
        )
        self.subtitle2 = TypographyStyle(
            fontWeight=600,
            lineHeight=22 / 14,
            fontSize=self.int_to_px(14)
        )
        self.body1 = TypographyStyle(
            lineHeight=1.5,
            fontSize=self.int_to_px(13)
        )
        self.body2 = TypographyStyle(
            lineHeight=22 / 14,
            fontSize=self.int_to_px(12)

        )
        self.caption = TypographyStyle(
            lineHeight=1.5,
            fontSize=self.int_to_px(11)
        )
        self.overline = TypographyStyle(
            fontWeight=700,
            lineHeight=1.5,
            fontSize=self.int_to_px(12),
            textTransform="uppercase"
        )
        self.button = TypographyStyle(
            fontWeight=700,
            lineHeight=24 / 14,
            fontSize=self.int_to_px(12),
            textTransform="unset"
        )

    @staticmethod
    def int_to_px(value: int) -> str:
        return f'{value}px'.encode('utf-8').decode('utf-8')
    
    @staticmethod
    def px_to_rem(value: int) -> str:
        """Chuyển đổi px sang rem."""
        return f"{value / 16}rem".encode('utf-8').decode('utf-8')

    def responsive_font_sizes(self, sm: int, md: int, lg: int) -> dict:
        """Tạo các kích thước phông chữ phản hồi cho các điểm dừng khác nhau."""
        return {
            '@media (min-width:600px)': {
                'fontSize': self.int_to_px(sm),
            },
            '@media (min-width:900px)': {
                'fontSize': self.int_to_px(md),
            },
            '@media (min-width:1200px)': {
                'fontSize': self.int_to_px(lg),
            },
        }



# # Sử dụng lớp Typography
# typography = Typography()

# # Truy xuất fontSize của h3
# print(typography.h3.fontSize)  # Trả về "1.5rem"
# print(typography.h3.fontWeight)  # Trả về "700"



from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState
    from qtmui.material.styles.create_theme.create_palette import Palette

from .properties_name import *
from ....system.color_manipulator import alpha


def checkbox(_theme) -> Dict:
    theme: ThemeState = _theme
    COLORS = ['primary', 'secondary', 'info', 'success', 'warning', 'error']
    lightMode = theme.palette.mode == 'light'
    theme_palette: Palette = theme.palette

    return {
        # CHECKBOX, RADIO, SWITCH
        'MuiFormControlLabel': {
            'styles': {
                'label': {
                    fontSize: theme.typography.body2.fontSize,
                    fontWeight: theme.typography.body2.fontWeight,
                    lineHeight: theme.typography.body2.lineHeight,
                }
            },
        },
        'MuiCheckbox': {
            'styles': {
                'root': lambda ownerState: {
                    **{
                        "width": "30px" if ownerState.get("size") == "small" else "36px",
                        "height": "30px" if ownerState.get("size") == "small" else "36px",
                    },
                    **{
                        "default": {
                            color: theme.palette.grey._900 if lightMode else theme.palette.common.white,
                            # p: theme.spacing,
                            p: '0px',
                            borderRadius: "15px" if ownerState.get("size") == "small" else "18px",
                            backgroundColor: "transparent",
                            "slots": {
                                "hover": {
                                    backgroundColor: alpha(theme_palette.grey._500, 0.24),
                                },
                                "checked": {
                                    backgroundColor: alpha(theme_palette.grey._500, 0.24),
                                },
                            }
                        },
                    },
                    **{
                        f"{_color}": {
                            color: getattr(theme.palette, _color).main,
                            # p: theme.spacing,
                            p: '0px',
                            borderRadius: "15px" if ownerState.get("size") == "small" else "18px",
                            backgroundColor: "transparent",
                            "slots": {
                                "hover": {
                                    backgroundColor: alpha(getattr(theme.palette, _color).main, 0.32),
                                },
                                "checked": {
                                    backgroundColor: alpha(getattr(theme.palette, _color).main, 0.32),
                                },
                            }
                        }
                        for _color in COLORS
                    }
                },
                "icon": lambda ownerState: {
                    color: theme.palette.text.disabled,
                    "size": {
                        "width": 20 if ownerState.get("size") == "small" else 26,
                        "height": 20 if ownerState.get("size") == "small" else 26,
                    }
                },
                "checkedIndicator": {
                    borderWidth: "1px",
                    borderRadius: "3px",
                    p: "8px",
                }
            }
        },
    }



from functools import wraps


import inspect
from typing import Callable, Optional, List
import hashlib
from functools import lru_cache, partial
from PySide6.QtCore import QObject, Signal
from .use_state import State, useState


def useCallback(callback: Callable = None, dependencies: Optional[List[State]] = []) -> Callable:
        """
        Khởi tạo state dựa trên giá trị tính từ callback (dựa trên dependencies)
        Và kết nối tín hiệu valueChanged của từng dependency để cập nhật state khi có thay đổi.
        """
        # Kiểm tra số lượng tham số của callback
        sig = inspect.signature(callback)
        # Tạo đối số mặc định nếu không có
        default_args = {
            k: (v.default if v.default is not inspect.Parameter.empty else None)
            for k, v in sig.parameters.items()
        }

        # value, setValue = useState(callback(**default_args))

        if isinstance(dependencies, list) and len(dependencies):
            for dep in dependencies:
                dep.valueChanged.connect(lambda default_args=default_args, *args, **kwargs: callback(**default_args))
        return callback


# def useCallback(callback, dependencies):
#     cache = {'value': None, 'dependencies': None}
    
#     @wraps(callback)
#     def wrapper(*args, **kwargs):
#         if cache['dependencies'] is None or cache['dependencies'] != dependencies:
#             cache['value'] = callback(*args, **kwargs)
#             cache['dependencies'] = dependencies[:]
#         return cache['value']
    
#     return wrapper




# # Giả lập các hằng số và đối tượng router
# LOGINPATHS = {'email': '/login/email', 'google': '/login/google'}
# router = {
#     'path': '/current/path',
#     'replace': lambda href: print(f"Redirecting to {href}"),
#     'setChecked': lambda value: print(f"Checked set to {value}")
# }

# # Trạng thái và phương pháp giả lập
# authenticated = False
# method = 'email'

# def check(authenticated, method, router):
#     if not authenticated:
#         searchParams = f"returnTo={router['path']}"
#         loginPath = LOGINPATHS[method]
#         href = f"{loginPath}?{searchParams}"
#         router['replace'](href)
#     else:
#         router['setChecked'](True)

# # Các dependencies mà hàm useCallback cần
# dependencies = [authenticated, method, router]

# # Gọi useCallback để tạo hàm được cache
# cachedCheck = useCallback(check, dependencies)

# # Sử dụng hàm được cache
# cachedCheck(authenticated, method, router)

# messesQuer(1, 2, 3, 4) 

# reduxThunk 
# async 




from redux.combine_reducers import combine_reducers
from .create_theme.theme_reducer import theme_reducer, ThemeState
from .location_reducer import location_reducer, LocationState

from redux import (
    BaseAction,
    BaseCombineReducerState,
    CombineReducerAction,
    CombineReducerRegisterAction,
    CombineReducerUnregisterAction,
    InitAction,
    InitializationActionError,
    Store,
    combine_reducers,
)
from redux.basic_types import (
    BaseEvent,
    CompleteReducerResult,
    FinishAction,
    ReducerResult,
)

# root_reducer, reducer_id = combine_reducers({
#     "products": product_reducer,
#     "users": user_reducer,
# })

class ProductAction(BaseAction): ...
class UserAction(BaseAction): ...


class StateType(BaseCombineReducerState):
    theme: ThemeState = None
    location: LocationState = None


# ActionType = InitAction | FinishAction | ProductAction | UserAction | CombineReducerAction
ActionType = InitAction | FinishAction | BaseAction | CombineReducerAction


root_reducer, root_reducer_id = combine_reducers(
    state_type=StateType,
    action_type=ActionType,  # pyright: ignore [reportArgumentType]
    theme=theme_reducer,
    location=location_reducer,
)

# store: Store[StateType] = Store(root_reducer)
# store._state.permission



import os

import pythoncom
import pywintypes
import win32api
import win32com
import win32com.client
import win32com.client.dynamic
import win32com.server.util
import win32timezone
import win32ui
from win32com import storagecon
from win32com.axcontrol import axcontrol
from win32com.test.util import CheckClean

S_OK = 0

now = win32timezone.now()


class LockBytes:
    _public_methods_ = [
        "ReadAt",
        "WriteAt",
        "Flush",
        "SetSize",
        "LockRegion",
        "UnlockRegion",
        "Stat",
    ]
    _com_interfaces_ = [pythoncom.IID_ILockBytes]

    def __init__(self, data=b""):
        self.data = data
        self.ctime = now
        self.mtime = now
        self.atime = now

    def ReadAt(self, offset, cb):
        print("ReadAt")
        result = self.data[offset : offset + cb]
        return result

    def WriteAt(self, offset, data):
        print("WriteAt " + str(offset))
        print("len " + str(len(data)))
        print("data:")
        # print(data)
        if len(self.data) >= offset:
            newdata = self.data[0:offset] + data
        print(len(newdata))
        if len(self.data) >= offset + len(data):
            newdata += self.data[offset + len(data) :]
        print(len(newdata))
        self.data = newdata
        return len(data)

    def Flush(self, whatsthis=0):
        print("Flush" + str(whatsthis))
        fname = os.path.join(win32api.GetTempPath(), "persist.doc")
        open(fname, "wb").write(self.data)
        return S_OK

    def SetSize(self, size):
        print("Set Size" + str(size))
        if size > len(self.data):
            self.data += b"\000" * (size - len(self.data))
        else:
            self.data = self.data[0:size]
        return S_OK

    def LockRegion(self, offset, size, locktype):
        print("LockRegion")

    def UnlockRegion(self, offset, size, locktype):
        print("UnlockRegion")

    def Stat(self, statflag):
        print("returning Stat " + str(statflag))
        return (
            "PyMemBytes",
            storagecon.STGTY_LOCKBYTES,
            len(self.data),
            self.mtime,
            self.ctime,
            self.atime,
            storagecon.STGM_DIRECT | storagecon.STGM_READWRITE | storagecon.STGM_CREATE,
            storagecon.STGM_SHARE_EXCLUSIVE,
            "{00020905-0000-0000-C000-000000000046}",
            0,  # statebits ?
            0,
        )


class OleClientSite:
    _public_methods_ = [
        "SaveObject",
        "GetMoniker",
        "GetContainer",
        "ShowObject",
        "OnShowWindow",
        "RequestNewObjectLayout",
    ]
    _com_interfaces_ = [axcontrol.IID_IOleClientSite]

    def __init__(self, data=""):
        self.IPersistStorage = None
        self.IStorage = None

    def SetIPersistStorage(self, IPersistStorage):
        self.IPersistStorage = IPersistStorage

    def SetIStorage(self, IStorage):
        self.IStorage = IStorage

    def SaveObject(self):
        print("SaveObject")
        if self.IPersistStorage is not None and self.IStorage is not None:
            self.IPersistStorage.Save(self.IStorage, 1)
            self.IStorage.Commit(0)
        return S_OK

    def GetMoniker(self, dwAssign, dwWhichMoniker):
        print("GetMoniker " + str(dwAssign) + " " + str(dwWhichMoniker))

    def GetContainer(self):
        print("GetContainer")

    def ShowObject(self):
        print("ShowObject")

    def OnShowWindow(self, fShow):
        print("ShowObject" + str(fShow))

    def RequestNewObjectLayout(self):
        print("RequestNewObjectLayout")


def test():
    # create a LockBytes object and
    # wrap it as a COM object
    #       import win32com.server.dispatcher
    lbcom = win32com.server.util.wrap(
        LockBytes(), pythoncom.IID_ILockBytes
    )  # , useDispatcher=win32com.server.dispatcher.DispatcherWin32trace)

    # create a structured storage on the ILockBytes object
    stcom = pythoncom.StgCreateDocfileOnILockBytes(
        lbcom,
        storagecon.STGM_DIRECT
        | storagecon.STGM_CREATE
        | storagecon.STGM_READWRITE
        | storagecon.STGM_SHARE_EXCLUSIVE,
        0,
    )

    # create our ClientSite
    ocs = OleClientSite()
    # wrap it as a COM object
    ocscom = win32com.server.util.wrap(ocs, axcontrol.IID_IOleClientSite)

    # create a Word OLE Document, connect it to our site and our storage
    oocom = axcontrol.OleCreate(
        "{00020906-0000-0000-C000-000000000046}",
        axcontrol.IID_IOleObject,
        0,
        (0,),
        ocscom,
        stcom,
    )

    mf = win32ui.GetMainFrame()
    hwnd = mf.GetSafeHwnd()

    # Set the host and document name
    # for unknown reason document name becomes hostname, and document name
    # is not set, debugged it, but don't know where the problem is?
    oocom.SetHostNames("OTPython", "This is Cool")

    # activate the OLE document
    oocom.DoVerb(-1, ocscom, 0, hwnd, mf.GetWindowRect())

    # set the hostnames again
    oocom.SetHostNames("OTPython2", "ThisisCool2")

    # get IDispatch of Word
    doc = win32com.client.Dispatch(oocom.QueryInterface(pythoncom.IID_IDispatch))

    # get IPersistStorage of Word
    dpcom = oocom.QueryInterface(pythoncom.IID_IPersistStorage)

    # let our ClientSite know the interfaces
    ocs.SetIPersistStorage(dpcom)
    ocs.SetIStorage(stcom)

    # use IDispatch to do the Office Word test
    # pasted from TestOffice.py

    wrange = doc.Range()
    for i in range(10):
        wrange.InsertAfter("Hello from Python %d\n" % i)
    paras = doc.Paragraphs
    for i in range(len(paras)):
        paras[i]().Font.ColorIndex = i + 1
        paras[i]().Font.Size = 12 + (4 * i)
    # XXX - note that
    # for para in paras:
    #       para().Font...
    # doesn't seem to work - no error, just doesn't work
    # Should check if it works for VB!

    dpcom.Save(stcom, 0)
    dpcom.HandsOffStorage()
    #       oocom.Close(axcontrol.OLECLOSE_NOSAVE) # or OLECLOSE_SAVEIFDIRTY, but it fails???

    # Save the ILockBytes data to "persist2.doc"
    lbcom.Flush()

    # exiting Winword will automatically update the ILockBytes data
    # and flush it to "%TEMP%\persist.doc"
    doc.Application.Quit()


if __name__ == "__main__":
    test()
    pythoncom.CoUninitialize()
    CheckClean()



from .button_base import ButtonBase
from .button import Button
from .button_group import ButtonGroup
from .button_group import ButtonGroup
from .toggle_button import ToggleButton
from .toggle_button_group import ToggleButtonGroup
from .fab import Fab
from .icon_button import IconButton
from .loading_button import LoadingButton
from .menu_button import MenuButton


from typing import Optional, List
from PySide6.QtWidgets import QLayout, QWidget, QLayoutItem, QSizePolicy, QFrame
from PySide6.QtCore import Qt, QRect, QSize, QPoint, QMargins

class FlowLayout(QLayout):
    def __init__(
            self,
            parent=None,
            alignment=Qt.AlignmentFlag.AlignCenter,
            flexWrap: str = "wrap",
            alignItems="center",
            justifyContent="space-between",
            children: Optional[List[QWidget]] = None,
            sx=None
    ):
        super().__init__(parent)

        self.setAlignment(Qt.AlignmentFlag.AlignBaseline)
        self._parent = parent
        self._item_list = []
        self._flexWrap = flexWrap
        self._justifyContent = justifyContent
        self._alignItems = alignItems
        self._children = children
        self._laying_out = False
        
        init_sx = {"flexBasic": None, "flexGrow": 0, "flexShrink": 1, "padding": "8px"}
        if sx is not None and isinstance(sx, dict):
            init_sx.update(sx)
        self._sx = init_sx
        self._current_rect_width = 0
        self._init_ui()

        self.setSizeConstraint(QLayout.SetMinimumSize)
        self._parent.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)

    def _init_ui(self):
        if self._parent is not None:
            margin = self.getPaddingFromSx()
            self.setContentsMargins(margin[0], margin[1], margin[2], margin[3])
            print(f"Debug: _init_ui - contentsMargins set to {margin}")
            if self._parent.parent():
                print(f"Debug: _init_ui - parent.parent() size = {self._parent.parent().size().width()}x{self._parent.parent().size().height()}")

        if self._children:
            for widget in self._children:
                if isinstance(widget, QWidget):
                    self.addWidget(widget)

    def __del__(self):
        item = self.takeAt(0)
        while item:
            item = self.takeAt(0)

    def addItem(self, item):
        if item:
            self._item_list.append(item)
            print(f"Debug: addItem - Added widget {item.widget()}")
            if self._parent.parent():
                print(f"Debug: addItem - parent.parent() size = {self._parent.parent().size().width()}x{self._parent.parent().size().height()}")
            self.invalidate()

    def count(self):
        return len(self._item_list)

    def itemAt(self, index):
        if 0 <= index < len(self._item_list):
            return self._item_list[index]
        return None

    def takeAt(self, index):
        if 0 <= index < len(self._item_list):
            return self._item_list.pop(index)
        return None

    def expandingDirections(self):
        return Qt.Orientation(Qt.Horizontal)

    def hasHeightForWidth(self):
        return True

    def heightForWidth(self, width):
        left, top, right, bottom = self.getContentsMargins()
        effective_width = max(0, width - left - right)
        height = self._do_layout(QRect(0, 0, effective_width, 0), True)
        print(f"Debug: heightForWidth - input width={width}, effective_width={effective_width}, height={height + top + bottom}")
        if self._parent.parent():
            print(f"Debug: heightForWidth - parent.parent() size = {self._parent.parent().size().width()}x{self._parent.parent().size().height()}")
        return height + top + bottom

    def setGeometry(self, rect):
        if self._laying_out:
            print("Debug: setGeometry - Skipping due to reentrant call")
            return
        self._current_rect_width = rect.width()
        super().setGeometry(rect)
        self._do_layout(rect, False)
        print(f"Debug: setGeometry - rect={rect}, width={rect.width()}, height={rect.height()}")
        if self._parent.parent():
            print(f"Debug: setGeometry - parent.parent() size = {self._parent.parent().size().width()}x{self._parent.parent().size().height()}")

    def sizeHint(self):
        return self.minimumSize()

    def getPaddingFromSx(self):
        padding = 8
        if self._sx and isinstance(self._sx, dict):
            padding_value = self._sx.get("padding", 8)
            if isinstance(padding_value, (int, float)):
                padding = padding_value
            elif isinstance(padding_value, str) and padding_value.endswith("px"):
                try:
                    padding = int(padding_value[:-2])
                except ValueError:
                    padding = 8
        paddingTop = self._sx.get("padding-top", padding)
        paddingBottom = self._sx.get("padding-bottom", padding)
        paddingLeft = self._sx.get("padding-left", padding)
        paddingRight = self._sx.get("padding-right", padding)
        print(f"Debug: getPaddingFromSx - padding=(left={paddingLeft}, top={paddingTop}, right={paddingRight}, bottom={paddingBottom})")
        return (paddingLeft, paddingTop, paddingRight, paddingBottom)

    def _calculate_unwrapped_width(self):
        total = 0
        spacing = self.spacing()
        for item in self._item_list:
            widget = item.widget()
            if hasattr(widget, 'layout') and isinstance(widget.layout(), FlowLayout):
                total += widget.layout()._calculate_unwrapped_width()
            else:
                total += item.sizeHint().width()
            total += spacing
        total -= spacing if len(self._item_list) > 0 else 0
        return total

    def minimumSize(self):
        left, top, right, bottom = self.getContentsMargins()
        min_width = 0
        max_height = 0
        total_width_no_wrap = self._calculate_unwrapped_width()
        for item in self._item_list:
            widget = item.widget()
            item_width = item.sizeHint().width()
            item_height = item.sizeHint().height()
            if hasattr(widget, 'layout') and isinstance(widget.layout(), FlowLayout):
                item_width = widget.layout().minimumWidth()
                item_height = widget.layout().heightForWidth(item_width)
            min_width = max(min_width, item_width)
            max_height = max(max_height, item_height)
        min_width = max(min_width, total_width_no_wrap // len(self._item_list) if len(self._item_list) > 0 else 0)
        size = QSize(min_width + left + right, max_height + top + bottom)
        print(f"Debug: minimumSize - size={size.width()}x{size.height()}, total_width_no_wrap={total_width_no_wrap}")
        return size

    def minimumWidth(self):
        left, _, right, _ = self.getContentsMargins()
        min_width = 0
        total_width_no_wrap = self._calculate_unwrapped_width()
        for item in self._item_list:
            widget = item.widget()
            item_width = item.sizeHint().width()
            if hasattr(widget, 'layout') and isinstance(widget.layout(), FlowLayout):
                item_width = widget.layout().minimumWidth()
            min_width = max(min_width, item_width)
        min_width = max(min_width, total_width_no_wrap // len(self._item_list) if len(self._item_list) > 0 else 0)
        print(f"Debug: minimumWidth - min_width={min_width + left + right}, total_width_no_wrap={total_width_no_wrap}")
        return min_width + left + right

    def maximumWidth(self):
        if hasattr(self, '_is_calculating_max_width') and self._is_calculating_max_width:
            print("Debug: maximumWidth - Skipping recursive call")
            return 0
        self._is_calculating_max_width = True
        left, _, right, _ = self.getContentsMargins()
        total_width = self._calculate_unwrapped_width()
        max_width = total_width + left + right
        self._is_calculating_max_width = False
        print(f"Debug: maximumWidth - max_width={max_width}")
        return max_width

    def _find_nearest_flow_layouts(self, widget):
        flow_layouts = []
        non_flow_width = 0
        if not widget:
            return flow_layouts, non_flow_width

        if hasattr(widget, 'layout') and isinstance(widget.layout(), FlowLayout):
            flow_layouts.append(widget.layout())
        else:
            non_flow_width += widget.sizeHint().width()
        return flow_layouts, non_flow_width

    def setAlignmentMode(self, align):
        if align in ["flex-start", "center", "flex-end", "space-between", "space-around", "space-evenly"]:
            self._justifyContent = align
        else:
            raise ValueError("Alignment must be one of 'flex-start', 'center', 'flex-end', 'space-between', 'space-around', 'space-evenly'")

    def setVerticalAlignmentMode(self, vertical_align):
        if vertical_align in ["flex-start", "center", "flex-end", "baseline", "stretch"]:
            self._alignItems = vertical_align
        else:
            raise ValueError("Vertical alignment must be one of 'flex-start', 'center', 'flex-end', 'baseline', 'stretch'")

    def _get_baseline_offset(self, item):
        widget = item.widget()
        if not widget:
            return 0
        width = widget.width() if widget.width() > 0 else item.sizeHint().width()
        height = item.sizeHint().height()
        if hasattr(widget, 'layout') and isinstance(widget.layout(), FlowLayout):
            height = widget.layout().heightForWidth(width)
        return height

    def _get_basis(self, item):
        widget = item.widget()
        basis = widget._sx.get("flexBasic") if hasattr(widget, '_sx') and widget._sx else None
        return basis if basis is not None else item.sizeHint().width()

    def _get_prop(self, item, name):
        widget = item.widget()
        val = widget._sx.get(name) if hasattr(widget, '_sx') and widget._sx else 0
        return int(val) if isinstance(val, (int, float)) else 0

    def _do_layout(self, rect, test_only):
        if self._laying_out:
            print("Debug: _do_layout - Skipping due to reentrant call")
            return 0
        self._laying_out = True

        left, top, right, bottom = self.getContentsMargins()
        effective_rect = QRect(rect.x() + left, rect.y() + top, rect.width() - left - right, max(0, rect.height() - top - bottom))
        x = effective_rect.x()
        y = effective_rect.y()
        line_height = 0
        spacing = self.spacing()
        line_widgets = []
        line_baseline_info = []
        available_width = effective_rect.width()
        current_line_width = 0
        temp_max_width = self._calculate_unwrapped_width()

        print(f"Debug: _do_layout - rect={rect}, effective_rect={effective_rect}, margins=(left={left}, top={top}, right={right}, bottom={bottom})")
        print(f"Debug: _do_layout - flow_layout available_width={available_width}, temp_max_width={temp_max_width}")
        if self._parent.parent():
            print(f"Debug: _do_layout - parent.parent() size = {self._parent.parent().size().width()}x{self._parent.parent().size().height()}")

        for item in self._item_list:
            widget = item.widget()
            widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)  # Giãn width cho widget con
            item_width = max(item.minimumSize().width(), available_width - current_line_width)
            item_height = item.sizeHint().height()
            if hasattr(widget, 'layout') and isinstance(widget.layout(), FlowLayout):
                sub_unwrapped_width = widget.layout()._calculate_unwrapped_width()
                item_width = max(widget.layout().minimumWidth(), min(available_width - current_line_width, sub_unwrapped_width))
                item_height = widget.layout().heightForWidth(item_width)
                if not test_only:
                    widget.layout()._do_layout(QRect(0, 0, item_width, item_height), False)

            next_x = current_line_width + item_width
            if self._flexWrap == "wrap" and next_x > available_width and line_widgets:
                self._align_line(line_widgets, line_baseline_info, effective_rect.x(), available_width, y, line_height, effective_rect.height(), test_only)
                y += line_height + spacing
                x = effective_rect.x()
                line_widgets = []
                line_baseline_info = []
                line_height = 0
                current_line_width = 0

            line_widgets.append((item, QPoint(x + current_line_width, y)))
            current_line_width += item_width + spacing
            line_height = max(line_height, item_height)
            line_baseline_info.append(item_height)

            print(f"Debug: _do_layout - widget {widget} at (x={x + current_line_width - item_width - spacing}, y={y}), width={item_width}, height={item_height}")

        if line_widgets:
            self._align_line(line_widgets, line_baseline_info, effective_rect.x(), available_width, y, line_height, effective_rect.height(), test_only)

        total_height = y + line_height - effective_rect.y()
        print(f"Debug: _do_layout - total_height={total_height}, effective_rect.height={effective_rect.height()}")

        if self._alignItems == "center" and not test_only:
            y_offset = (effective_rect.height() - total_height) // 2 if effective_rect.height() > total_height else 0
            for item in self._item_list:
                geom = item.geometry()
                item.setGeometry(geom.translated(0, y_offset))
                print(f"Debug: _do_layout - Adjusted widget {item.widget()} to y={geom.y() + y_offset}")

        self._laying_out = False
        return total_height

    def _align_line(self, line_widgets, baseline_offsets, start_x, total_width, y, line_height, total_height, test_only):
        spacing = self.spacing()
        count = len(line_widgets)
        total_spacing = spacing * (count - 1)
        line_width = 0
        for item, _ in line_widgets:
            line_width += item.sizeHint().width()
        line_width += total_spacing

        offset_x = 0
        extra_space = 0
        if self._justifyContent == "center":
            offset_x = (total_width - line_width) // 2 if total_width >= line_width else 0
        elif self._justifyContent == "flex-end":
            offset_x = total_width - line_width if total_width >= line_width else 0
        elif self._justifyContent == "space-between" and count > 1:
            extra_space = (total_width - line_width) // (count - 1) if total_width >= line_width else 0
        elif self._justifyContent == "space-around" and count > 0:
            extra_space = (total_width - line_width) // (count * 2) if total_width >= line_width else 0
            offset_x = extra_space
        elif self._justifyContent == "space-evenly" and count > 0:
            extra_space = (total_width - line_width) // (count + 1) if total_width >= line_width else 0
            offset_x = extra_space

        current_x = start_x + offset_x
        for i, (item, _) in enumerate(line_widgets):
            widget = item.widget()
            width = item.sizeHint().width()
            height = item.sizeHint().height()
            if hasattr(widget, 'layout') and isinstance(widget.layout(), FlowLayout):
                width = widget.layout().maximumWidth()
                height = widget.layout().heightForWidth(width)

            if self._alignItems == "center":
                offset_y = (line_height - height) // 2
            elif self._alignItems == "flex-end":
                offset_y = line_height - height
            else:
                offset_y = 0

            if not test_only:
                item.setGeometry(QRect(QPoint(current_x, y + offset_y), QSize(width, height)))
                print(f"Debug: _align_line - widget {widget} set to geometry=(x={current_x}, y={y + offset_y}, width={width}, height={height})")

            current_x += width + spacing + extra_space

class FlowContainer(QFrame):
    def __init__(self, children):
        super().__init__()
        self.layout = FlowLayout(self, sx={"padding": "8px"})
        self.setLayout(self.layout)
        if not isinstance(children, list):
            raise TypeError(f"Argument 'children' has incorrect type (expected list, got {type(children)})")
        for widget in children:
            self.layout.addWidget(widget)


import time
from functools import wraps

def timer(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        start_time = time.time()           # Ghi lại thời điểm bắt đầu
        result = func(*args, **kwargs)     # Thực hiện hàm
        end_time = time.time()             # Ghi lại thời điểm kết thúc
        elapsed = end_time - start_time    # Tính thời gian chạy
        print(f"_______________Function______________ '{func.__name__}' executed in {elapsed:.4f} seconds")
        return result
    return wrapper


import uuid

from PySide6.QtWidgets import QStackedWidget, QStackedLayout, QWidget, QVBoxLayout, QLabel
from PySide6.QtCore import Property, Slot
from ...material.hook_form.rhf_masonry_radio_group import RHFMasonryRadioGroup
from qtmui.material.styles import useTheme

styles = '''   
    QFrame  {{
        border-radius: {};
        background-color:{};
    }}

    QFrame:hover  {{
    }}
'''

class RHFStackedRadioGroup(QStackedWidget):
    """
    Box
    Base container

    Args:
        stackingMode: QStackedLayout
        children: list[QWidget]

    Returns:
        new instance of PySyde6.QtWidgets.QStackedWidget
    """
    def __init__(self,  
                 name: str,
                 stackingMode: QStackedLayout = None,
                 children: list[QWidget] = None,
        ):
        super().__init__()
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        self._name = name
        self._value = None

        if isinstance(children, list) and len(children) > 0:
            for widget in children:
                self.addWidget(widget)

        if stackingMode:
            self.layout().setStackingMode(stackingMode)


        self.lbl_helper_text = QLabel(self)
        self.lbl_helper_text.setFixedHeight(14)
        self.lbl_helper_text.setStyleSheet('padding-left: 8px;')
        self.lbl_helper_text.setText("sssssssss")
        self.layout().addWidget(self.lbl_helper_text)

        # self.setup_ui()

    def show_child(self, type, id):
        for item in self.findChildren(type):
            if hasattr(item, "_id"):
                if item._id == id:
                    self.setCurrentWidget(item)

    def setup_ui(self):
        visible_count = 0
        for item in self.findChildren(RHFMasonryRadioGroup):
            item.valueChanged.connect(self.set_value)
            if item._value is None:
                pass
                # raise AttributeError("Must contain checked=True in one of the elements of the options property.")
            if item._isVisible:
                self.setCurrentWidget(item)
                self._value = item._value
                visible_count += 1
        if visible_count > 1:
            pass
            # raise AttributeError("At least one AutoComplete must have the isVisible attribute set to True.")

    def set_value(self, value):
        print('show_even  ===> set_value')
        self._value = value

    @Property(bool)
    def stateSignal(self):
        return self._stateSignal

    @stateSignal.setter
    def stateSignal(self, value):
        self._stateSignal = value
        self._stateSignal.connect(self.state)


    @Slot(object)
    def state(self, state):
        if self._name == state.get("field"):
            if state.get("error"):
                self.set_state({"status": "invalid", "message": state.get("error_message")})
                self.lbl_helper_text.setText(str(state.get("error_message")))
                self.lbl_helper_text.setStyleSheet(f'padding-left: 8px;color: {useTheme().palette.error.main};')


import uuid
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QFrame, QVBoxLayout, QWidget, QSizePolicy

from qtmui.hooks import State, useEffect
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.utils.validate_params import _validate_param
from qtmui.material.widget_base import PyWidgetBase
from qtmui.configs import LOAD_WIDGET_ASYNC

class List(QFrame, PyWidgetBase):
    """
    A component that renders a list of items, styled like Material-UI List.

    The `List` component is used to group and display a collection of items, with support for
    subheaders, dense layouts, and customizable padding and styles.

    Parameters
    ----------
    ariaLabelledby : State or str, optional
        The ID of the element that labels the list for accessibility. Default is None.
        Can be a `State` object for dynamic updates.
    children : State, QWidget, List[Union[QWidget, str]], or None, optional
        The content of the list (widgets, text, or list of widgets/text). Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State or str, optional
        The component used for the root node (e.g., "QFrame"). Default is None (uses QFrame).
        Can be a `State` object for dynamic updates.
    dense : State or bool, optional
        If True, uses compact vertical padding for keyboard/mouse input. Default is False.
        Can be a `State` object for dynamic updates.
    disablePadding : State or bool, optional
        If True, removes vertical padding from the list. Default is False.
        Can be a `State` object for dynamic updates.
    subheader : State or QWidget, optional
        The content of the subheader (e.g., ListSubheader). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class,
        supporting props of the native component (e.g., parent, style, className).

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - The `children` prop supports widgets, text (rendered as Typography), or lists of widgets/text.
    - The `dense` prop affects child components via context, reducing spacing and margins.

    Demos:
    - List: https://qtmui.com/material-ui/qtmui-list/

    API Reference:
    - List API: https://qtmui.com/material-ui/api/list/
    """

    def __init__(
        self,
        ariaLabelledby: Optional[Union[State, str, Callable]] = None,
        children: Optional[Union[State, QWidget, List[Union[QWidget, str]]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, str]] = None,
        dense: Union[State, bool] = False,
        disablePadding: Union[State, bool] = False,
        subheader: Optional[Union[State, QWidget]] = None,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        **kwargs
    ):
        super().__init__()
        self.setObjectName(f"List-{str(uuid.uuid4())}")
        PyWidgetBase._setUpUi(self)

        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_ariaLabelledby(ariaLabelledby)
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_dense(dense)
        self._set_disablePadding(disablePadding)
        self._set_subheader(subheader)
        self._set_sx(sx)

        self._init_ui()


    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.list", param_name="ariaLabelledby", supported_signatures=Union[State, Callable, str, type(None)])
    def _set_ariaLabelledby(self, value):
        """Assign value to ariaLabelledby."""
        self._ariaLabelledby = value

    def _get_ariaLabelledby(self):
        """Get the ariaLabelledby value."""
        return self._ariaLabelledby.value if isinstance(self._ariaLabelledby, State) else self._ariaLabelledby

    @_validate_param(file_path="qtmui.material.list", param_name="children", supported_signatures=Union[State, QWidget, List, type(None)])
    def _set_children(self, value):
        """Assign value to children and store references."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.list", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.list", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.list", param_name="dense", supported_signatures=Union[State, bool])
    def _set_dense(self, value):
        """Assign value to dense."""
        self._dense = value

    def _get_dense(self):
        """Get the dense value."""
        return self._dense.value if isinstance(self._dense, State) else self._dense

    @_validate_param(file_path="qtmui.material.list", param_name="disablePadding", supported_signatures=Union[State, bool])
    def _set_disablePadding(self, value):
        """Assign value to disablePadding."""
        self._disablePadding = value

    def _get_disablePadding(self):
        """Get the disablePadding value."""
        return self._disablePadding.value if isinstance(self._disablePadding, State) else self._disablePadding

    @_validate_param(file_path="qtmui.material.list", param_name="subheader", supported_signatures=Union[State, QWidget, type(None)])
    def _set_subheader(self, value):
        """Assign value to subheader."""
        self._subheader = value
        if isinstance(value, QWidget):
            self._widget_references.append(value)

    def _get_subheader(self):
        """Get the subheader value."""
        return self._subheader.value if isinstance(self._subheader, State) else self._subheader

    @_validate_param(file_path="qtmui.material.list", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx


    def _init_ui(self):
        # Thiết lập layout chính cho List
        self._layout = QVBoxLayout(self)
        self._layout.setContentsMargins(0, 0, 0, 0)  # Bỏ các lề mặc định
        self.setLayout(self._layout)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)

        # Áp dụng subheader nếu có
        if self._subheader:
            self._layout.addWidget(self._subheader)

        # Nếu prop `disablePadding` được đặt thành True, xóa padding
        if self._disablePadding:
            self._layout.setSpacing(0)


        # Nếu prop `dense` được đặt thành True, sử dụng padding nhỏ hơn
        elif self._dense:
            self._layout.setSpacing(3)  # Padding nhỏ hơn so với mặc định

        # Thêm các children (nếu có)
        if self._children:
            if isinstance(self._children, list):
                for index, widget in enumerate(self._children):
                    if LOAD_WIDGET_ASYNC:
                        self._do_task_async(lambda index=index, widget=widget: self._layout.insertWidget(index, widget))
                    else:
                        self._layout.insertWidget(index, widget)
                
        self.theme = useTheme()
        
        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        
        self._set_stylesheet()
                
    def _set_stylesheet(self, component_styles=None):
        theme = useTheme()

        if not component_styles:
            component_styles = theme.components
        
        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx)
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx)
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        self.setStyleSheet(
            f"""
                List {{
                    {sx_qss}
                }}
            """
        )


from __future__ import annotations
from dataclasses import dataclass
from typing import Any, Callable, Dict, List, Optional, Tuple

from PySide6.QtCore import QRect, QSize, Qt, QTimer, QEvent
from PySide6.QtWidgets import QWidget, QLayout, QLayoutItem

# tái dùng engine đã test pass
from .flexbox_engine import StackLayoutEngine, StackItem


LayoutCallback = Callable[[Dict[str, Any]], None]


def _sx_of_widget(w: QWidget) -> Dict[str, Any]:
    sx = getattr(w, "_sx_dict", None)
    return sx if isinstance(sx, dict) else {}


def _safe_item_sizes(w: QWidget) -> Tuple[QSize, QSize]:
    sh = w.sizeHint()
    msh = w.minimumSizeHint()

    sw = sh.width() if sh.width() > 0 else msh.width()
    shh = sh.height() if sh.height() > 0 else msh.height()

    if sw <= 0:
        sw = 1
    if shh <= 0:
        shh = 1

    size_hint = QSize(sw, shh)

    mw = max(0, int(w.minimumWidth()))
    mh = max(0, int(w.minimumHeight()))
    if mw <= 0 and msh.width() > 0:
        mw = msh.width()
    if mh <= 0 and msh.height() > 0:
        mh = msh.height()
    min_size = QSize(mw, mh)
    return size_hint, min_size


def _parse_item_sx(sx: Dict[str, Any]) -> Tuple[int, int, int, Optional[str]]:
    order = int(sx.get("order", 0) or 0)
    fg = int(sx.get("flexGrow", 0) or 0)
    fs = int(sx.get("flexShrink", 1) or 1)
    al = sx.get("alignSelf", None)
    al = str(al) if isinstance(al, str) else None
    return order, fg, fs, al


def _summarize_lines_by_y(rects: Dict[str, QRect], keys_in_order: List[str]) -> List[List[str]]:
    # group theo y giống test_case_011
    lines: List[List[str]] = []
    y_to_line: Dict[int, List[str]] = {}
    for k in keys_in_order:
        r = rects.get(k)
        if not r:
            continue
        y_to_line.setdefault(r.y(), []).append(k)
    for y in sorted(y_to_line.keys()):
        lines.append(y_to_line[y])
    return lines


class _FlexEngineLayout(QLayout):
    """
    QLayout "mỏng": chỉ bridge Qt layout lifecycle -> StackLayoutEngine.compute
    Không container widget, setGeometry trực tiếp cho QLayoutItem.
    """

    def __init__(self, owner: "FlexBox", parent: Optional[QWidget] = None):
        super().__init__(parent)
        self._owner = owner
        self._items: List[QLayoutItem] = []
        self.engine = StackLayoutEngine()

        self.setSpacing(0)

    # ---- QLayout required ----
    def addItem(self, item: QLayoutItem) -> None:
        self._items.append(item)
        self.invalidate()

    def count(self) -> int:
        return len(self._items)

    def itemAt(self, index: int) -> Optional[QLayoutItem]:
        return self._items[index] if 0 <= index < len(self._items) else None

    def takeAt(self, index: int) -> Optional[QLayoutItem]:
        if 0 <= index < len(self._items):
            it = self._items.pop(index)
            self.invalidate()
            return it
        return None

    def hasHeightForWidth(self) -> bool:
        return True

    def heightForWidth(self, w: int) -> int:
        if w <= 0 or not self._items:
            return 0
        props = self._owner._props_dict()
        stack_items = self._to_stack_items()
        return self.engine.measure_height_for_width(w, stack_items, props)

    def sizeHint(self) -> QSize:
        return self.minimumSize()

    def minimumSize(self) -> QSize:
        # tối thiểu gọn: max(min_size) + margins
        mw = 0
        mh = 0
        for it in self._items:
            w = it.widget()
            if not w:
                continue
            _, mn = _safe_item_sizes(w)
            mw = max(mw, mn.width())
            mh = max(mh, mn.height())
        l, t, r, b = self.getContentsMargins()
        return QSize(mw + l + r, mh + t + b)

    def setGeometry(self, rect: QRect) -> None:
        super().setGeometry(rect)
        if not self._items:
            return

        props = self._owner._props_dict()
        stack_items = self._to_stack_items()

        rects = self.engine.compute(rect, stack_items, props)

        # apply geometry
        for it in self._items:
            w = it.widget()
            if not w:
                continue
            key = self._key_for_widget(w)
            r = rects.get(key)
            if r is not None:
                it.setGeometry(r)

        # notify owner (for logging/testing)
        self._owner._emit_layout(rect, rects)

    # ---- internal ----
    def _key_for_widget(self, w: QWidget) -> str:
        # ổn định theo vị trí trong layout (index) + id, tránh trùng
        idx = -1
        for i, it in enumerate(self._items):
            if it.widget() is w:
                idx = i
                break
        name = w.objectName()
        base = name if (name and name != "Stack") else f"{idx}"
        return f"{base}#{id(w)}"

    def _to_stack_items(self) -> List[StackItem]:
        out: List[StackItem] = []
        for it in self._items:
            w = it.widget()
            if not w:
                continue
            size_hint, min_size = _safe_item_sizes(w)
            sx = _sx_of_widget(w)
            order, fg, fs, al = _parse_item_sx(sx)
            out.append(
                StackItem(
                    key=self._key_for_widget(w),
                    size_hint=size_hint,
                    min_size=min_size,
                    flex_grow=max(0, int(fg)),
                    flex_shrink=max(0, int(fs)),
                    order=int(order),
                    align_self=al,
                )
            )
        return out


class FlexBox(QWidget):
    """
    FlexBox = QWidget dùng engine flexbox (StackLayoutEngine) qua QLayout.
    - children=[...] trong constructor là đủ (không setup ngoài)
    - sx={...} chứa props flex
    - onLayout(payload) để lấy rects/lines/breakpoints
    """

    def __init__(
        self,
        *,
        sx: Optional[Dict[str, Any]] = None,
        children: Optional[List[QWidget]] = None,
        onLayout: Optional[LayoutCallback] = None,
        parent: Optional[QWidget] = None,
    ):
        super().__init__(parent)
        self._sx = sx if isinstance(sx, dict) else {}
        self._onLayout = onLayout

        self._layout_pending = False
        self._last_payload_sig: Optional[Tuple[int, int]] = None

        lay = _FlexEngineLayout(owner=self, parent=self)
        self.setLayout(lay)

        # margins/padding sẽ nằm trong props, nhưng Qt layout margins cũng cần 0 để engine toàn quyền
        self.layout().setContentsMargins(0, 0, 0, 0)

        if children:
            for w in children:
                self.addChild(w)

        # request initial layout (coalesce)
        self.requestLayout()

    def addChild(self, w: QWidget) -> None:
        # mọi thứ gọn trong tree: addChild chỉ dùng nội bộ constructor/updates
        self.layout().addWidget(w)
        self.requestLayout()

    def setSx(self, sx: Dict[str, Any]) -> None:
        self._sx = sx if isinstance(sx, dict) else {}
        self.requestLayout()

    def requestLayout(self) -> None:
        if self._layout_pending:
            return
        self._layout_pending = True
        QTimer.singleShot(0, self._do_layout)

    def _do_layout(self) -> None:
        self._layout_pending = False
        # Qt sẽ gọi setGeometry khi cần; nhưng đôi khi cần kích hoạt lại
        self.updateGeometry()
        self.layout().activate()

    def event(self, e: QEvent):
        # khi show/resize, đảm bảo layout cập nhật
        if e.type() in (QEvent.Show, QEvent.Resize, QEvent.Polish):
            self.requestLayout()
        return super().event(e)

    # ---- props + emit ----
    def _props_dict(self) -> Dict[str, Any]:
        # normalize sx giống MUI naming
        gap = self._sx.get("gap", 0)
        padding = self._sx.get("padding", 0)

        # padding: nếu dùng hệ spacing scale 8px, bạn có thể đổi ở đây.
        # hiện lấy trực tiếp số px (giống các test).
        return {
            "direction": self._sx.get("flexDirection", "row"),
            "flexWrap": self._sx.get("flexWrap", "nowrap"),
            "justifyContent": self._sx.get("justifyContent", "flex-start"),
            "alignItems": self._sx.get("alignItems", "stretch"),
            "alignContent": self._sx.get("alignContent", "flex-start"),
            "gap": int(gap),
            "padding": int(padding),
        }

    def _emit_layout(self, outer_rect: QRect, item_rects: Dict[str, QRect]) -> None:
        if not self._onLayout:
            return

        # stable ordered keys by layout order (index in keys string starts with idx)
        keys = list(item_rects.keys())

        # line grouping theo y của items (cho parent)
        lines = _summarize_lines_by_y(item_rects, keys)

        payload = {
            "rect": outer_rect,
            "items": item_rects,
            "lines": lines,
            "props": self._props_dict(),
        }

        # tránh spam: signature theo (w,h)
        sig = (outer_rect.width(), outer_rect.height())
        if self._last_payload_sig == sig:
            # vẫn có thể thay đổi lines khi wrap, nên không chặn tuyệt đối
            pass
        self._last_payload_sig = sig

        self._onLayout(payload)



from ...system.color_manipulator import alpha
from .palette import palette
from .create_palette import Palette

class Shadows:
    z1 : tuple = None
    z2 : tuple = None
    z3 : tuple = None
    z4 : tuple = None
    z5 : tuple = None
    z6 : tuple = None
    z7 : tuple = None
    z8 : tuple = None
    z9 : tuple = None
    z10 : tuple = None
    z11 : tuple = None
    z12 : tuple = None
    z13 : tuple = None
    z14 : tuple = None
    z15 : tuple = None
    z16 : tuple = None
    z17 : tuple = None
    z18 : tuple = None
    z19 : tuple = None
    z20 : tuple = None
    z21 : tuple = None
    z22 : tuple = None
    z23 : tuple = None
    z24 : tuple = None

    primary : tuple = ""
    info : tuple = ""
    secondary : tuple = ""
    success : tuple = ""
    warning : tuple = ""
    error : tuple = ""
    card : tuple = ""
    dialog : tuple = ""
    dropdown : tuple = ""
    

    def __init__(self,options):
        self.z1 = options['z1']
        self.z2 = options['z2']
        self.z3 = options['z3']
        self.z4 = options['z4']
        self.z5 = options['z5']
        self.z6 = options['z6']
        self.z7 = options['z7']
        self.z8 = options['z8']
        self.z9 = options['z9']
        self.z10 = options['z10']
        self.z11 = options['z11']
        self.z12 = options['z12']
        self.z13 = options['z13']
        self.z14 = options['z14']
        self.z15 = options['z15']
        self.z16 = options['z16']
        self.z17 = options['z17']
        self.z18 = options['z18']
        self.z19 = options['z19']
        self.z20 = options['z20']
        self.z21 = options['z21']
        self.z22 = options['z22']
        self.z23 = options['z23']
        self.z24 = options['z24']
        self.primary = options['primary']
        self.info = options['info']
        self.secondary = options['secondary']
        self.success = options['success']
        self.warning = options['warning']
        self.error = options['error']
        self.card = options['card']
        self.dialog = options['dialog']
        self.dropdown = options['dropdown']




def create_shadows(palette: Palette)->Shadows:
    transparent = alpha(palette.grey._200 if palette.mode == "light" else palette.grey._200, 0.04)
    transparent = palette.grey._300
    card_color = alpha(palette.grey._500 if palette.mode == "light" else palette.common.black, 0.2)
    dialog_dropdown_color = alpha(palette.grey._500 if palette.mode == "light" else palette.common.black, 0.24)

    shadows_1_24 = {}
    for i in range(25):
        if i != 0:
            shadows_1_24.update({f"z{i}": (i, 0, 1, int(f"{i*2 + 25}"))})

    shadows = shadows_1_24 | {
        # "primary" : (10, 8, 16, f"{alpha(palette.primary.main, 0.24)}"),
        # "info" : (10, 8, 16, f"{alpha(palette.info.main, 0.24)}"),
        # "secondary" : (10, 8, 16, f"{alpha(palette.secondary.main, 0.24)}"),
        # "success" : (10, 8, 16, f"{alpha(palette.success.main, 0.24)}"),
        # "warning" : (10, 8, 16, f"{alpha(palette.warning.main, 0.24)}"),
        # "error" : (10, 8, 16, f"{alpha(palette.error.main, 0.24)}"),

        # "primary" : (10, 0, 1, f"{alpha(palette.primary.main, 0.24)}"),
        # "info" : (10, 0, 1, f"{alpha(palette.info.main, 0.24)}"),
        # "secondary" : (10, 0, 1, f"{alpha(palette.secondary.main, 0.24)}"),
        # "success" : (10, 0, 1, f"{alpha(palette.success.main, 0.24)}"),
        # "warning" : (10, 0, 1, f"{alpha(palette.warning.main, 0.24)}"),
        # "error" : (10, 0, 1, f"{alpha(palette.error.main, 0.24)}"),

        "primary" : (10, 0, 1, palette.primary.main),
        "info" : (10, 0, 1, palette.info.main),
        "secondary" : (10, 0, 1, palette.secondary.main),
        "success" : (10, 0, 1, palette.success.main),
        "warning" : (10, 0, 1, palette.warning.main),
        "error" : (10, 0, 1, palette.error.main),

        "card" : (0, 0, 2, 25),
        "dialog" : (10, 40, 80, 25),
        "dropdown" : (0, 0, 1, 25)
    }

    return Shadows(shadows)






import uuid
from typing import Optional, Callable, Any, Dict, Union

from PySide6.QtWidgets import QWidget
from PySide6.QtCore import QObject,Qt,QEvent
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

class MuiInputBase:
    def __init__(
        self,
        parent=None,
        onMouseEnter: Optional[Callable] = None,  
        onMouseLeave: Optional[Callable] = None,  
        onMousePress: Optional[Callable] = None,  
        onMouseRelease: Optional[Callable] = None,  
        onFocusIn: Optional[Callable] = None,  
        onFocusOut: Optional[Callable] = None,  
        **kwargs
    ):
        self._onMouseEnter = onMouseEnter
        self._onMouseLeave = onMouseLeave
        self._onMousePress = onMousePress
        self._onMouseRelease = onMouseRelease
        self._onFocusIn = onFocusIn
        self._onFocusOut = onFocusOut

    
    def enterEvent(self, event):
        if self._onMouseEnter:
            self._onMouseEnter()
    
    def leaveEvent(self, event):
        if self._onMouseLeave:
            self._onMouseLeave()
    
    def mousePressEvent(self, event):
        if self._onMousePress:
            self._onMousePress()
    
    def mouseReleaseEvent(self, event):
        if self._onMouseRelease:
            self._onMouseRelease()
    
    def focusInEvent(self, event) -> None:
        if self._onFocusIn:
            self._onFocusIn(event)

    def focusOutEvent(self, event) -> None:
        if self._onFocusOut:
            self._onFocusOut(event)



from typing import Optional, Type, TypeVar, Dict, Any, Callable, Union

# Giả định ButtonBaseActions là một lớp với các phương thức cụ thể
class ButtonBaseActions:
    def focusVisible(self):
        print("focusVisible method called")

# Giả định TouchRippleActions là một lớp với các phương thức cụ thể
class TouchRippleActions:
    pass

# Giả định TouchRippleProps là một từ điển các thuộc tính
TouchRippleProps = Dict[str, Any]

# Giả định ButtonBaseClasses là một từ điển các thuộc tính
ButtonBaseClasses = Dict[str, Any]

# Giả định SxProps là một từ điển các thuộc tính
SxProps = Dict[str, Any]

# Giả định Theme là một lớp với các thuộc tính cụ thể
class Theme:
    pass

# Định nghĩa ButtonBaseTypeMap như một lớp
class ButtonBaseTypeMap:
    def __init__(self,
                 additional_props: Optional[Dict[str, Any]] = None,
                 default_component: Optional[Any] = 'button'):
        self.props = additional_props if additional_props else {}
        self.defaultComponent = default_component
        self.props.update({
            "action": None,
            "centerRipple": False,
            "children": None,
            "classes": None,
            "disabled": False,
            "disableRipple": False,
            "disableTouchRipple": False,
            "focusRipple": False,
            "focusVisibleClassName": None,
            "LinkComponent": 'a',
            "onFocusVisible": None,
            "sx": None,
            "tabIndex": 0,
            "TouchRippleProps": None,
            "touchRippleRef": None
        })

# Định nghĩa ExtendButtonBaseTypeMap như một lớp
class ExtendButtonBaseTypeMap:
    def __init__(self, type_map: Type[ButtonBaseTypeMap]):
        self.props = type_map.props
        self.defaultComponent = type_map.defaultComponent

# Định nghĩa một hàm giả lập ExtendButtonBase
def ExtendButtonBase(type_map: Type[ButtonBaseTypeMap]) -> Callable[..., Any]:
    class ButtonBaseImpl(ExtendButtonBaseTypeMap):
        def __init__(self, href: str, **override_props):
            super().__init__(type_map)
            self.href = href
            self.override_props = override_props

        def render(self):
            # Logic render giả lập
            print(f"Rendering button with href: {self.href} and props: {self.override_props}")

    return ButtonBaseImpl

# Định nghĩa ButtonBaseProps như một lớp kế thừa
class ButtonBaseProps(ButtonBaseTypeMap):
    def __init__(self,
                 root_component: Optional[Type] = None,
                 additional_props: Optional[Dict[str, Any]] = None,
                 **kwargs):
        super().__init__(additional_props=additional_props, default_component=root_component)
        self.props.update(kwargs)
        self.component = root_component

# Ví dụ sử dụng lớp
button_base_props = ButtonBaseProps(
    root_component='button',
    additional_props={'extraProp': 'extraValue'},
    centerRipple=True,
    disabled=True
)

print(button_base_props.props)



import asyncio
from functools import lru_cache
import threading
from typing import Callable, Dict, List, Optional, Union

from PySide6.QtWidgets import QHBoxLayout, QWidget
from PySide6.QtCore import Qt, Signal, QTimer

from qtmui.hooks import State
from qtmui.utils.data import deep_merge

from ..system.color_manipulator import alpha

from ..button.button_base import ButtonBase
from .list_item_checkbox import ListItemCheckbox
from ..utils.validate_params import _validate_param

from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style


class ListItemButton(ButtonBase):
    """
    A component that renders a clickable button within a ListItem, styled like Material-UI ListItemButton.

    The `ListItemButton` component extends ButtonBase and is used to create interactive list items with
    support for selection, dense layouts, dividers, and customizable alignment and padding.

    Parameters
    ----------
    alignItems : State or str, optional
        Defines the align-items style property ("center", "flex-start"). Default is "center".
        Can be a `State` object for dynamic updates.
    autoFocus : State or bool, optional
        If True, the list item is focused during the first mount. Default is False.
        Can be a `State` object for dynamic updates.
    children : State, QWidget, List[Union[QWidget, str]], or None, optional
        The content of the list item (widgets, text, or list of widgets/text). Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State or str, optional
        The component used for the root node (e.g., "QFrame"). Default is None (uses QFrame).
        Can be a `State` object for dynamic updates.
    dense : State or bool, optional
        If True, uses compact vertical padding. Default is False (inherits from parent List).
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the component is disabled. Default is False.
        Can be a `State` object for dynamic updates.
    disableGutters : State or bool, optional
        If True, removes left and right padding. Default is False.
        Can be a `State` object for dynamic updates.
    divider : State or bool, optional
        If True, adds a 1px light border to the bottom. Default is False.
        Can be a `State` object for dynamic updates.
    focusVisibleClassName : State or str, optional
        Class name applied when the element gains focus via keyboard. Default is None.
        Can be a `State` object for dynamic updates.
    key : State or str, optional
        The key for the list item, used for identification in lists. Default is None.
        Can be a `State` object for dynamic updates.
    minHeight : State or int, optional
        The minimum height of the list item. Default is None.
        Can be a `State` object for dynamic updates.
    onClick : State or Callable, optional
        Callback function triggered when the list item is clicked. Default is None.
        Can be a `State` object for dynamic updates.
    selected : State or bool, optional
        If True, applies selected styling. Default is False.
        Can be a `State` object for dynamic updates.
    selectedKey : State or str, optional
        The key of the currently selected item, used to determine selection state. Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    size : State or str, optional
        The size of the list item ("small", "medium", "large"). Default is "medium".
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `ButtonBase` class,
        supporting props of ButtonBase and native component (e.g., parent, style, className).

    Attributes
    ----------
    VALID_ALIGN_ITEMS : list[str]
        Valid values for `alignItems`: ["center", "flex-start"].
    VALID_SIZES : list[str]
        Valid values for `size`: ["small", "medium", "large"].

    Signals
    -------
    themeChanged : Signal
        Emitted when the theme changes.

    Notes
    -----
    - Props of the ButtonBase component are supported via `**kwargs` (e.g., `onClick`, `disabled`, `focusVisible`).
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - The `children` prop supports widgets, text (rendered as Typography), or lists of widgets/text.
    - The `dense` prop defaults to the value inherited from the parent List component.

    Demos:
    - ListItemButton: https://qtmui.com/material-ui/qtmui-listitembutton/

    API Reference:
    - ListItemButton API: https://qtmui.com/material-ui/api/list-item-button/
    """
    themeChanged = Signal()
    
    updateStyleSheet = Signal(object)

    VALID_ALIGN_ITEMS = ["center", "flex-start"]
    VALID_SIZES = ["small", "medium", "large"]

    def __init__(self, 
                alignItems: Union[State, str] = "center",
                autoFocus: Union[State, bool] = False,
                children: Optional[Union[State, QWidget, List[Union[QWidget, str]]]] = None,
                classes: Optional[Union[State, Dict]] = None,
                component: Optional[Union[State, str]] = None,
                dense: Union[State, bool] = False,
                disabled: Union[State, bool] = False,
                disableGutters: Union[State, bool] = False,
                divider: Union[State, bool] = False,
                focusVisibleClassName: Optional[Union[State, str]] = None,
                key: Optional[Union[State, str]] = None,
                minHeight: Optional[Union[State, int]] = None,
                onClick: Optional[Union[State, Callable]] = None,
                selected: Union[State, bool] = False,
                selectedKey: Optional[Union[State, str]] = None,
                sx: Optional[Union[State, List, Dict, Callable, str]] = None,
                size: Union[State, str] = "medium",
                asynRenderQss: Optional[Union[State, bool]] = False,
                
                 **kwargs):
        super().__init__()
        self.setObjectName(str(id(self)))
        if sx:
            self._setSx(sx)
            
        self._kwargs = {
            **kwargs,
            "size": size,
        }
            
        self._setKwargs(self._kwargs)
        
        self._setUpUi()

        # Gán các props thành thuộc tính của class
        self.kwargs = kwargs

        # Set properties with validation
        self._set_alignItems(alignItems)
        self._set_autoFocus(autoFocus)
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_dense(dense)
        self._set_disabled(disabled)
        self._set_disableGutters(disableGutters)
        self._set_divider(divider)
        self._set_focusVisibleClassName(focusVisibleClassName)
        self._set_key(key)
        self._set_minHeight(minHeight)
        self._set_onClick(onClick)
        self._validate_selected(selected)
        self._set_selectedKey(selectedKey)
        self._set_sx(sx)
        self._set_size(size)

        self._asynRenderQss = asynRenderQss

        self.__init_ui()

    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.list_item_button", param_name="alignItems", supported_signatures=Union[State, str], valid_values=VALID_ALIGN_ITEMS)
    def _set_alignItems(self, value):
        """Assign value to alignItems."""
        self._alignItems = value

    def _get_alignItems(self):
        """Get the alignItems value."""
        return self._alignItems.value if isinstance(self._alignItems, State) else self._alignItems

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="autoFocus", supported_signatures=Union[State, bool])
    def _set_autoFocus(self, value):
        """Assign value to autoFocus."""
        self._autoFocus = value

    def _get_autoFocus(self):
        """Get the autoFocus value."""
        return self._autoFocus.value if isinstance(self._autoFocus, State) else self._autoFocus

    # @_validate_param(file_path="qtmui.material.list_item_button", param_name="children", supported_signatures=Union[State, QWidget, List, type(None)])
    def _set_children(self, value):
        """Assign value to children and store references."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="dense", supported_signatures=Union[State, bool])
    def _set_dense(self, value):
        """Assign value to dense."""
        self._dense = value

    def _get_dense(self):
        """Get the dense value."""
        return self._dense.value if isinstance(self._dense, State) else self._dense

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="disabled", supported_signatures=Union[State, bool])
    def _set_disabled(self, value):
        """Assign value to disabled."""
        self._disabled = value
        self.setDisabled(self._get_disabled())

    def _get_disabled(self):
        """Get the disabled value."""
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="disableGutters", supported_signatures=Union[State, bool])
    def _set_disableGutters(self, value):
        """Assign value to disableGutters."""
        self._disableGutters = value

    def _get_disableGutters(self):
        """Get the disableGutters value."""
        return self._disableGutters.value if isinstance(self._disableGutters, State) else self._disableGutters

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="divider", supported_signatures=Union[State, bool])
    def _set_divider(self, value):
        """Assign value to divider."""
        self._divider = value

    def _get_divider(self):
        """Get the divider value."""
        return self._divider.value if isinstance(self._divider, State) else self._divider

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="focusVisibleClassName", supported_signatures=Union[State, str, type(None)])
    def _set_focusVisibleClassName(self, value):
        """Assign value to focusVisibleClassName."""
        self._focusVisibleClassName = value

    def _get_focusVisibleClassName(self):
        """Get the focusVisibleClassName value."""
        return self._focusVisibleClassName.value if isinstance(self._focusVisibleClassName, State) else self._focusVisibleClassName

    # @_validate_param(file_path="qtmui.material.list_item_button", param_name="key", supported_signatures=Union[State, str, type(None)])
    def _set_key(self, value):
        """Assign value to key."""
        self._key = value

    def _get_key(self):
        """Get the key value."""
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="minHeight", supported_signatures=Union[State, int, type(None)])
    def _set_minHeight(self, value):
        """Assign value to minHeight."""
        self._minHeight = value

    def _get_minHeight(self):
        """Get the minHeight value."""
        return self._minHeight.value if isinstance(self._minHeight, State) else self._minHeight

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="onClick", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClick(self, value):
        """Assign value to onClick."""
        self._onClick = value

    def _get_onClick(self):
        """Get the onClick value."""
        return self._onClick.value if isinstance(self._onClick, State) else self._onClick

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="selected", supported_signatures=Union[State, bool])
    def _validate_selected(self, value=None):
        """Assign value to selected based on selected or selectedKey."""
        self._selected = value

    def _get_selected(self):
        """Get the selected value."""
        return self._selected.value if isinstance(self._selected, State) else self._selected

    # @_validate_param(file_path="qtmui.material.list_item_button", param_name="selectedKey", supported_signatures=Union[State, str, type(None)])
    def _set_selectedKey(self, value):
        """Assign value to selectedKey."""
        self._selectedKey = value

    def _get_selectedKey(self):
        """Get the selectedKey value."""
        return self._selectedKey.value if isinstance(self._selectedKey, State) else self._selectedKey

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.list_item_button", param_name="size", supported_signatures=Union[State, str], valid_values=VALID_SIZES)
    def _set_size(self, value):
        """Assign value to size."""
        self._size = value

    def _get_size(self):
        """Get the size value."""
        return self._size.value if isinstance(self._size, State) else self._size

    def __init_ui(self):
        
        self.setDisabled(self._disabled)

        self.setLayout(QHBoxLayout())
        # self.layout().setContentsMargins(6,6,6,6)
        self.layout().setContentsMargins(0,0,0,0)
        # self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)

        self.setCursor(Qt.CursorShape.PointingHandCursor)


        if self._minHeight:
            self.setMinimumHeight(self._minHeight)

        if self._selectedKey:
            if isinstance(self._selectedKey, State):
                self._selectedKey.valueChanged.connect(self._set_selected)
                
        if isinstance(self._selected, State):
            self._selected.valueChanged.connect(self._set_selected)

        self._set_selected()

        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            
            for child in self._children:
                if hasattr(child, '_secondary'):
                    if getattr(child, '_secondary'):
                        self.setFixedHeight(48)

                if isinstance(child, QWidget):
                    self.layout().addWidget(child)
                    
        self.theme = useTheme()

        self.theme.state.valueChanged.connect(self._onThemeChanged)
        # QTimer.singleShot(0, self._scheduleSetStyleSheet)
        if self._asynRenderQss:
            self.updateStyleSheet.connect(self._updateStylesheet)
        else:
            self._setStyleSheet()
        
        self.destroyed.connect(lambda obj: self._onDestroy())


    def _onDestroy(self, obj=None):
        # Cancel task nếu đang chạy
        if hasattr(self, "_setupStyleSheet") and self._setupStyleSheet and not self._setupStyleSheet.done():
            self._setupStyleSheet.cancel()

    def _onThemeChanged(self):
        if not self.isVisible():
            return
        QTimer.singleShot(0, self._scheduleSetStyleSheet)

    def _scheduleSetStyleSheet(self):
        self._setupStyleSheet = asyncio.ensure_future(self._lazy_setStyleSheet())

    async def _lazy_setStyleSheet(self):
        self._setStyleSheet()

    @classmethod
    @lru_cache(maxsize=128)
    def _getStyleSheet(cls, objectName: str, styledConfig: str="ListItemButton"):
        theme = useTheme()
        if hasattr(cls, "styledDict"):
            themeComponent = deep_merge(theme.components, cls.styledDict)
        else:
            themeComponent = theme.components
            
        PyListItemButton_root = themeComponent["PyListItemButton"].get("styles")["root"](cls.ownerState)
        PyListItemButton_root_qss = get_qss_style(PyListItemButton_root)
        
        PyListItemButton_root_slot_hover_qss = get_qss_style(PyListItemButton_root["slots"]["hover"])
        PyListItemButton_root_slot_selected_qss = get_qss_style(PyListItemButton_root["slots"]["selected"])
        PyListItemButton_root_slot_selected_hover_qss = get_qss_style(PyListItemButton_root["slots"]["selected"]["hover"])
        PyListItemButton_root_props_divider_qss = get_qss_style(PyListItemButton_root["props"]["divider"])
        PyListItemButton_root_props_disableGutters_qss = get_qss_style(PyListItemButton_root["props"]["disableGutters"])

        PyListItemButton_root_props_active_qss = ""
        PyListItemButton_root_props_active_slot_hover_qss = ""
        if PyListItemButton_root["props"].get("active"):
            cls.setProperty("active", True)
            PyListItemButton_root_props_active_qss = get_qss_style(PyListItemButton_root["props"]["active"])
            PyListItemButton_root_props_active_slot_hover_qss = get_qss_style(PyListItemButton_root["props"]["active"]["slots"]["hover"])

        stylesheet = f"""
                #{objectName} {{
                    {PyListItemButton_root_qss}
                }}
                #{objectName}:hover {{
                    {PyListItemButton_root_slot_hover_qss}
                }}

                #{objectName}[selected=true] {{
                    {PyListItemButton_root_slot_selected_qss}
                }}
                #{objectName}[selected=true]:hover {{
                    {PyListItemButton_root_slot_selected_hover_qss}
                }}
                #{objectName}[divider=true] {{
                    {PyListItemButton_root_props_divider_qss}
                }}
                #{objectName}[disableGutters=true] {{
                    {PyListItemButton_root_props_disableGutters_qss}
                }}
                #{objectName}[active=true] {{
                    {PyListItemButton_root_props_active_qss}
                }}
                #{objectName}[active=true]:hover {{
                    {PyListItemButton_root_props_active_slot_hover_qss}
                }}
                
                """
        
        return stylesheet
    
    def _renderStylesheet(self):
        stylesheet = ""
        if hasattr(self, "styledDict"):
            root = self.styledDict.get("PyListItemButton", {}).get("styles", {}).get("root", None)(self._kwargs)
            if root:
                stylesheet = self._getStyleSheet(objectName=self.objectName(), styledConfig=str(root))
        else:
            stylesheet = self._getStyleSheet(objectName=self.objectName())
            
        sxQss = ""
        if self._sx:
            sxQss = self._getSxQss(sxStr=str(self._sx), className=f"#{self.objectName()}")
            # sxQss = self._getSxQss(sxStr=str(self._sx), className=f"ListItemButton")

        stylesheet = f"""
            {stylesheet}
            {sxQss}
        """
        
        self.updateStyleSheet.emit(stylesheet)
        
    @classmethod
    def _setSx(cls, sx: dict = {}):
        cls.sxDict = sx
        
    @classmethod
    def _setKwargs(cls, kwargs: dict = {}):
        cls.ownerState = kwargs

    @classmethod
    @lru_cache(maxsize=128)
    def _getSxQss(cls, sxStr: str = "", className: str = "PyWidgetBase"):
        sx_qss = get_qss_style(cls.sxDict, class_name=className)
        return sx_qss
        
    def _updateStylesheet(self, stylesheet):
        self.setStyleSheet(stylesheet)
    
    def _setStyleSheet(self):
        
        # Thêm viền nếu divider=True
        if self._get_selected():
            self.setProperty("selected", "true")
        else:
            self.setProperty("selected", "false")
            
        if self._divider:
            self.setProperty("divider", "true")
        if self._disableGutters:
            self.setProperty("disableGutters", "true")
        
        stylesheet = ""
        if hasattr(self, "styledDict"):
            root = self.styledDict.get("PyListItemButton", {}).get("styles", {}).get("root", None)(self._kwargs)
            if root:
                stylesheet = self._getStyleSheet(objectName=self.objectName(), styledConfig=str(root))
        else:
            stylesheet = self._getStyleSheet(objectName=self.objectName())
            
        sxQss = ""
        if self._sx:
            sxQss = self._getSxQss(sxStr=str(self._sx), className=f"#{self.objectName()}")
            # sxQss = self._getSxQss(sxStr=str(self._sx), className=f"ListItemButton")

        stylesheet = f"""
            {stylesheet}
            {sxQss}
        """
        
        self.setStyleSheet(stylesheet)


    def _set_selected_key(self, value=None):
        if len(self.findChildren(ListItemCheckbox)):
            item_checkbox: ListItemCheckbox = self.findChildren(ListItemCheckbox)[0]
            item_checkbox.setAttribute(Qt.WA_TransparentForMouseEvents)

            if self._key == 0:
                pass
            
            if isinstance(self._selectedKey, State):
                if self._key in self._selectedKey.value:
                    item_checkbox._checkbox.set_checked(True)
                else:
                    item_checkbox._checkbox.set_checked(False)
        else:
            if isinstance(self._selectedKey, State):
                if self._selectedKey.value == self._key:
                    self._selected = True
                else:
                    self._selected = False
            elif self._selectedKey:
                if self._selectedKey == self._key:
                    self._selected = True
                else:
                    self._selected = False
            self._setStyleSheet()

    def _set_selected(self, value=None):
        if len(self.findChildren(ListItemCheckbox)):
            item_checkbox: ListItemCheckbox = self.findChildren(ListItemCheckbox)[0]
            item_checkbox.setAttribute(Qt.WA_TransparentForMouseEvents)

            if self._key == 0:
                pass
            
            if isinstance(self._selectedKey, State):
                if self._key in self._selectedKey.value:
                    item_checkbox._checkbox.set_checked(True)
                else:
                    item_checkbox._checkbox.set_checked(False)

        self._setStyleSheet()


    def _get_align_items_style(self, alignItems):
        """Thiết lập style cho align-items"""
        if alignItems == 'center':
            return "text-align: center;"
        elif alignItems == 'flex-start':
            return "text-align: left;"
        return ""

    def focusInEvent(self, event):
        """Quản lý lớp khi element được focus"""
        super().focusInEvent(event)
        if self._focusVisibleClassName:
            self.setStyleSheet(self.styleSheet() + f" {self._focusVisibleClassName} {{ outline: 1px solid #000; }}")

    def mouseReleaseEvent(self, event):
        if self._onClick:
            try:
                self._onClick(self._key)
            except Exception as e:
                self._onClick()
        return super().mouseReleaseEvent(event)

    def showEvent(self, event):
        if self._asynRenderQss:
            threading.Thread(target=self._renderStylesheet, args=(), daemon=True).start()
        return super().showEvent(event)



from typing import Optional, Callable




# from ...utils.data import deep_merge

class StyledOptions:
    shouldForwardProp: Optional[str] 
    label: Optional[str] 
    name: Optional[str] 
    slot: Optional[str] 
    overridesResolver: Optional[Callable] 
    skipVariantsResolver: Optional[bool] 
    skipSx: Optional[bool] 

def styled(component, options: dict={}, styleFn: Callable=None):
    """
    Tạo ra một styled component dựa trên component gốc và hàm styleFn.
    
    styleFn có thể là một hàm nhận vào (props, theme) và trả về một dictionary chứa style,
    hoặc đơn giản là một dictionary.
    """

    shouldForwardProp = options.get("shouldForwardProp") 
    label = options.get("label") 
    name = options.get("name") 
    slot = options.get("slot") 
    overridesResolver = options.get("overridesResolver") 
    skipVariantsResolver = options.get("skipVariantsResolver") 
    skipSx = options.get("skipSx") 


    class StyledComponent(component):
        def __init__(self, *args, **kwargs):
            # Lưu props nếu cần
            self.props = kwargs.copy()
            self._setProps(kwargs)
            
            # if shouldForwardProp:
            #     for key, value in self.props.items():
            #         if shouldForwardProp(key):
            #             self.setProperty(key, value)
            #             self.style().unpolish(self)
            #             self.style().polish(self)
            #             self.update()
            
            self._setStyledDict(styleFn=styleFn)
            super().__init__(*args, **kwargs)

            if styleFn:
                if shouldForwardProp:
                    for key, value in self.props.items():
                        if shouldForwardProp(key):
                            self.setProperty(key, value)
                            self.style().unpolish(self)
                            self.style().polish(self)
                            self.update()

                # self.setProperty("styled")
                # self.setProperty("overidde")
                # self.setProperty("sx")

                # Chuyển đổi style thành chuỗi QSS và áp dụng
                # if hasattr(self, "component_styles"):
                #     component_styles = deep_merge(self.component_styles, styleFn())
                #     self._set_stylesheet(component_styles)

        @classmethod
        def _setProps(cls, props={}):
            cls.props = props
            
        @classmethod
        def _setStyledDict(cls, styleFn: Callable=None):
            if isinstance(styleFn, Callable):
                cls.styledDict = styleFn()
            else:
                cls.styledDict = {}

    return StyledComponent




import inspect
from typing import Callable, List
from .use_state import State

def useEffect(callback: Callable, dependencies: List[State]) -> None:
    """
    Hook useEffect nhận vào một danh sách các dependency (State).
    Mỗi khi bất kỳ dependency nào thay đổi (phát tín hiệu valueChanged),
    callback sẽ được gọi.

    Nếu callback không nhận tham số, nó được gọi mà không có đối số.
    Nếu callback nhận ít nhất một tham số, một danh sách các giá trị hiện tại của các dependency
    sẽ được truyền vào.
    """
    try:
        sig = inspect.signature(callback)
    except (ValueError, TypeError):
        sig = None
    # Tạo đối số mặc định nếu không có
    default_args = {}
    if sig is not None and len(sig.parameters) > 0:
        default_args = {
            k: (v.default if v.default is not inspect.Parameter.empty else None)
            for k, v in sig.parameters.items()
        }
    for dep in dependencies:
        if isinstance(dep, State):
            dep.valueChanged.connect(lambda *args, **kwargs: callback(**default_args))
        else:
            raise ValueError("Dependencies only accepted State")



import inspect
from typing import Callable, Optional, Union, List, Dict
from dataclasses import field, replace
from functools import lru_cache

from PySide6.QtWidgets import QFrame, QVBoxLayout
from PySide6.QtCore import QTimer

from redux.main import Store

# from qtmui.hooks.use_effect import useEffect
# from qtmui.hooks import useState

from PySide6.QtWidgets import QFrame

from qtmui.material.styles.create_theme.components.qss_properties import QssProperties

from ._generateButtonQss import generateButtonQss

from ...material.styles.create_theme.theme_reducer import CreateThemeAction, ChangePaletteAction, MergeOverideComponentsAction, ThemeState

from .root_reducer import root_reducer, StateType, ActionType


from PySide6.QtCore import QObject, Property, Signal

from qtmui.utils.lodash import merge

from ..system import (
    hexToRgb,
    rgbToHex,
    hslToRgb,
    decomposeColor,
    recomposeColor,
    getContrastRatio,
    getLuminance,
    emphasize,
    alpha,
    darken,
    lighten,
)

from .styled import styled

class ThemeSignal(QObject):
    valueChanged = Signal()

    def __init__(self, theme=None):
        super().__init__()
        self._theme = theme

    def getTheme(self) -> dict:
        return self._theme

    def setTheme(self, value):
        if self._theme != value:
            self._theme = value
            self.valueChanged.emit()

    theme = Property(str, getTheme, setTheme, notify=valueChanged)

themeSignal = ThemeSignal()

def onThemeChanged(data: dict):
    themeSignal.theme = data
    

theme_store: Store[StateType, ActionType, None] = Store(root_reducer)
theme_store.dispatch(CreateThemeAction())


def setTheme(mode):
    theme_store.dispatch(ChangePaletteAction(mode=mode))
    onThemeChanged(theme_store._state.theme)

# @lru_cache(maxsize=128) ===> lỗi 

def useTheme():
    theme: ThemeState = theme_store._state.theme
    theme = replace(theme, state=themeSignal)
    return theme



class ThemeProvider(QFrame):
    def __init__(
        self, 
        children: Optional[List],
        theme: Optional[Dict]
    ):
        super().__init__()
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        
        self._children = children
        
        # themeSignal.changed.connect(self.initUi)
        
        theme_store.dispatch(MergeOverideComponentsAction(payload=theme))
        
        QTimer.singleShot(0, self.initUi)
        

        
        def setSt(mode):
            if mode == "dark":
                stylesheet = ""
                with open("[type=Button][variant=Contained][color=Default][mode=Dark].qss", "r", encoding="utf-8") as f:
                    stylesheet += f.read()
                with open("[type=Button][variant=Contained][color=Primary][mode=Dark].qss", "r", encoding="utf-8") as f:
                    stylesheet += f.read()
                
                self.setStyleSheet(stylesheet)
            else:
                stylesheet = ""
                with open("[type=Button][variant=Contained][color=Default][mode=Light].qss", "r", encoding="utf-8") as f:
                    stylesheet += f.read()
                with open("[type=Button][variant=Contained][color=Primary][mode=Light].qss", "r", encoding="utf-8") as f:
                    stylesheet += f.read()
                self.setStyleSheet(stylesheet)
        
        @theme_store.autorun(lambda state: state.theme)
        def _theme(_theme: ThemeState):
            print("_theme changed__________________________1111111111", _theme.palette.primary.main)
            # themeSignal.changed.emit()
            # setSt(_theme.palette.mode)
            # QTimer.singleShot(0, self.initUi) # Cách này khi load nhiều widget sẽ bị crash
            """
                self.setCentralWidget(
                    ThemeProvider(
                        theme={
                            "palette": {
                                "primary": {
                                    "main": "#007FFF",
                                    "dark": "#0066CC",
                                }
                            }
                        },
                        children=lambda: [ # Nếu không có lambda thì Box tạo trước khi hàm __initUi được của ThemeProvider được gọi
                            Box(
                                sx={
                                    "width": "100px",
                                    "height": "10px",
                                    "border-radius": 1,
                                    "background-color": "primary.main",
                                    "&:hover": {
                                        "background-color": "primary.dark",
                                    },
                                }
                            )
                            for i in range(3000)
                        ],
                    )
                )
            """
            
        
    def initUi(self):
        
        # button_styles = generateButtonQss(theme_store._state.theme, get_qss_style)
        # print("button_styles generated__________________________", button_styles)
        print("initUi called__________________________55555555555555555")
        
        if isinstance(self._children, Callable):
            self._children = self._children()
                
        if isinstance(self._children, list):
            for widget in self._children:
                self.layout().addWidget(widget)
            


###################### test gen qss file #########################
def camel_to_kebab_case(s: str) -> str:
    """
    Chuyển đổi chuỗi từ camelCase sang kebab-case.
    """
    return ''.join(['-' + c.lower() if c.isupper() else c for c in s]).lstrip('-')


def resolve_theme_value(theme, value: Union[str, Callable], key: str=None) -> Optional[str]:
    """
    Giải quyết giá trị theme từ chuỗi dạng 'palette.primary.main', 'text.secondary', v.v.
    """

    if isinstance(value, str) and (
        value.startswith("text.") 
        or value.startswith("background.")
        or value.startswith("primary.")
        or value.startswith("secondary.")
        or value.startswith("info.")
        or value.startswith("warning.")
        or value.startswith("error.")
        or value.startswith("grey.")
        ):
        value = f"palette.{value}"
        # print(value)

    if isinstance(value, str) and value.startswith("palette."):
        parts = value.split(".")
        theme_value = theme
        _value = None
        for part in parts:
            theme_value = getattr(theme_value, part, None)
            if theme_value is None:
                return None
        return theme_value
    elif isinstance(value, Callable):
        # return value(theme) # cũ 
        
        sig = inspect.signature(value)
        # Tạo đối số mặc định nếu không có
        default_args = {
            k: (v.default if v.default is not inspect.Parameter.empty else None)
            for k, v in sig.parameters.items()
        }

        try:
            return value(**default_args)
        except Exception as e:
            return None
    # elif "font" not in key and ((isinstance(value, int) or isinstance(value, float)) and 0 <= value <= 3):
    elif "font" not in key and ((isinstance(value, int) or isinstance(value, float)) and 0 <= value <= 5):
    # elif "font" not in key and ((isinstance(value, int) or isinstance(value, float))): # bị vỡ hết
        # value = 8 * value
        value = value * theme.spacing.default_spacing

    return value


def get_qss_style(sx: Dict, class_name: Optional[str] = None) -> str:
    """
    Hàm tạo chuỗi QSS từ dictionary sx, hỗ trợ nested selectors và pseudo-classes.
    Nếu class_name=None, chỉ trả về chuỗi thuộc tính mà không bao bọc trong block.
    """
    # print("get_qss_style_____________________", sx)
    if isinstance(sx, str):
        return sx

    theme = useTheme()
    base_styles = []  # Styles cấp cao
    nested_styles = []  # Styles cho các selector lồng nhau

    for key, value in sx.items():
        # Chỉ áp dụng camel_to_kebab_case cho các key không bắt đầu bằng "&"
        if not key.startswith("&"):
            css_key = camel_to_kebab_case(key)
        else:
            css_key = key

        # Xử lý các selector lồng nhau (bắt đầu bằng "&")
        if css_key.startswith("&"):
            print('nested selector found_____________________', css_key)
            selector = css_key[1:].strip()  # Bỏ "&" và khoảng trắng
            if class_name is None:
                continue  # Bỏ qua nested selectors nếu không có class_name
            if selector.startswith(":"):  # Pseudo-class như &:hover
                pseudo_part, *child_part = selector.split(" ", 1)
                pseudo = pseudo_part[1:]  # Bỏ ":"
                if child_part:  # Có selector con, ví dụ &:hover .Typography
                    nested_selector = f"{class_name}[slot={pseudo}] {child_part[0].lstrip('.')}"
                else:
                    nested_selector = f"{class_name}[slot={pseudo}]"
            else:  # Không có pseudo-class, ví dụ & Typography hoặc & #container
                if " " in selector:  # Có khoảng trắng, ví dụ & Typography
                    nested_selector = selector.split(" ", 1)[1]  # Lấy phần sau khoảng trắng
                else:
                    nested_selector = f"{class_name} {selector}"

            print('nested _____________________', nested_selector)

            if isinstance(value, dict):
                nested_qss = get_qss_style(value, nested_selector)
                nested_styles.append(nested_qss)
            continue

        # Xử lý thuộc tính CSS cấp cao
        
        resolved_value = resolve_theme_value(theme, value, key)
        if resolved_value is None:
            continue

        if css_key in QssProperties.__dict__.values():

            if css_key in ["marginX", "mx"]:
                base_styles.append(f"margin-left: {resolved_value};")
                base_styles.append(f"margin-right: {resolved_value};")
            elif css_key in ["marginY", "my"]:
                base_styles.append(f"margin-top: {resolved_value};")
                base_styles.append(f"margin-bottom: {resolved_value};")
            elif css_key in ["paddingX", "px"]:
                base_styles.append(f"padding-left: {resolved_value};")
                base_styles.append(f"padding-right: {resolved_value};")
            elif css_key in ["paddingY", "py"]:
                base_styles.append(f"padding-top: {resolved_value};")
                base_styles.append(f"padding-bottom: {resolved_value};")
            elif css_key == "width" and str(resolved_value).find("%") == -1:
                base_styles.append(f"min-width: {resolved_value};")
                base_styles.append(f"max-width: {resolved_value};")
            elif css_key == "height" and str(resolved_value).find("%") == -1:
                base_styles.append(f"min-height: {resolved_value};")
                base_styles.append(f"max-height: {resolved_value};")
            elif css_key not in ["mb", "display", "flex-shrink", "flex-grow", "align-items", 
                                "flex-direction", "align-self", "z-index", "justify-content"]:
                base_styles.append(f"{css_key}: {resolved_value};")

    # Tạo QSS
    qss = ""
    if base_styles:
        if class_name:
            qss = f"{class_name} {{\n    " + "\n    ".join(base_styles) + "\n}}"
        else:
            qss = "\n    ".join(base_styles)  # Chỉ trả về chuỗi thuộc tính nếu không có class_name
    if nested_styles:
        if qss:
            qss += "\n\n"
        qss += "\n\n".join(nested_styles)

    return qss.strip().replace("}}", "}")


from dataclasses import dataclass, field
from typing import Optional, Dict, Any, Callable, Union

from .shape import ShapeOptions, Shape
from .create_breakpoints import BreakpointsOptions, Breakpoints
from .create_spacing import SpacingOptions, Spacing
from ..style_function_sx.style_function_sx import SxProps
from ..style_function_sx.default_sx_config import SxConfig
from .apply_styles import ApplyStyles

class Direction:
    ltr = "ltr"
    rtl = "rtl"
# Direction = Union['ltr', 'rtl']


# ThemeOptions dataclass equivalent
@dataclass
class ThemeOptions:
    shape: Optional[ShapeOptions] = None
    breakpoints: Optional[BreakpointsOptions] = None
    direction: Optional[Direction] = 'ltr'
    mixins: Optional[Any] = None
    palette: Optional[Dict[str, Any]] = field(default_factory=dict)
    shadows: Optional[Any] = None
    spacing: Optional[SpacingOptions] = None
    transitions: Optional[Any] = None
    components: Optional[Dict[str, Any]] = field(default_factory=dict)
    typography: Optional[Any] = None
    zIndex: Optional[Dict[str, int]] = field(default_factory=dict)
    unstable_sxConfig: Optional[SxConfig] = None

# Theme dataclass equivalent
@dataclass
class Theme:
    shape: Shape
    breakpoints: Breakpoints
    direction: Direction
    palette: Dict[str, Any]
    shadows: Optional[Any] = None
    spacing: Spacing = field(default_factory=Spacing)
    transitions: Optional[Any] = None
    components: Optional[Dict[str, Any]] = field(default_factory=dict)
    mixins: Optional[Any] = None
    typography: Optional[Any] = None
    zIndex: Optional[Dict[str, int]] = field(default_factory=dict)
    applyStyles: Optional[Callable[[str], ApplyStyles]] = None
    unstable_sxConfig: Optional[SxConfig] = None
    unstable_sx: Optional[Callable[[SxProps], Dict[str, Any]]] = None


def createTheme(options: Optional[ThemeOptions] = None, *args: Any) -> Theme:
    options = options or ThemeOptions()
    
    # Create Shape, Breakpoints, and other components
    shape = Shape() if options.shape is None else options.shape
    breakpoints = Breakpoints() if options.breakpoints is None else options.breakpoints
    
    palette = options.palette or {"mode": "light"}
    direction = options.direction or "ltr"
    shadows = options.shadows
    spacing = Spacing() if options.spacing is None else options.spacing
    typography = options.typography
    zIndex = options.zIndex or {}
    components = options.components or {}
    
    # Create and return the full Theme object
    return Theme(
        shape=shape,
        breakpoints=breakpoints,
        direction=direction,
        palette=palette,
        shadows=shadows,
        spacing=spacing,
        typography=typography,
        zIndex=zIndex,
        components=components,
        applyStyles=None,  # Can be updated later
        unstable_sxConfig=options.unstable_sxConfig,
        unstable_sx=None  # Can be updated later
    )


import uuid

from PySide6.QtWidgets import QVBoxLayout, QFrame

from ..textfield import TextField


from qtmui.qss_name import *

class OutlinedInput(QFrame):
    def __init__(
            self,
            *args,
            **kwargs
            ):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))
        # self.setStyleSheet('''#{}  {{ {} }}'''.format(self.objectName(), "background: red;")) # str multi line
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.layout().addWidget(TextField(*args, **kwargs))
        self.setStyleSheet("background: red;") # str multi line


